# -*- coding: utf-8 -*-
from collective.volto.blocksfield.field import BlocksField
from design.plone.contenttypes import _
from plone.app.contenttypes.interfaces import INewsItem
from plone.app.dexterity import textindexer
from plone.app.z3cform.widget import RelatedItemsFieldWidget
from plone.autoform import directives as form
from plone.autoform.interfaces import IFormFieldProvider
from plone.supermodel import model
from z3c.relationfield.schema import RelationChoice
from z3c.relationfield.schema import RelationList
from zope import schema
from zope.component import adapter
from zope.interface import implementer
from zope.interface import provider


@provider(IFormFieldProvider)
class INewsAdditionalFieldsBase(model.Schema):
    numero_progressivo_cs = schema.TextLine(
        title=_(
            "numero_progressivo_cs_label",
            default="Numero progressivo del comunicato stampa",
        ),
        required=False,
    )

    a_cura_di = RelationList(
        title=_("a_cura_di_label", default="A cura di"),
        description=_(
            "a_cura_di_help",
            default="Seleziona l'ufficio di comunicazione responsabile di "
            "questa notizia/comunicato stampa.",
        ),
        required=True,
        default=[],
        value_type=RelationChoice(vocabulary="plone.app.vocabularies.Catalog"),
    )

    a_cura_di_persone = RelationList(
        title=_("a_cura_di_persone_label", default="Persone"),
        description=_(
            "a_cura_di_persone_help",
            default="Seleziona una lista di persone dell'amministrazione "
            "citate in questa notizia/comunicato stampa. Questa informazione "
            'verrà mostrata nella sezione "A cura di".',
        ),
        default=[],
        value_type=RelationChoice(vocabulary="plone.app.vocabularies.Catalog"),
        required=False,
    )

    luoghi_correlati = RelationList(
        title=_("luoghi_correlati_label", default="Luoghi correlati"),
        description=_(
            "luoghi_correlati_help",
            default="Seleziona una lista di luoghi citati.",
        ),
        default=[],
        value_type=RelationChoice(vocabulary="plone.app.vocabularies.Catalog"),
        required=False,
    )

    notizie_correlate = RelationList(
        title=_("notizie_correlate_label", default="Notizie correlate"),
        description=_(
            "notizie_correlate_help",
            default="Seleziona una lista di notizie correlate a questa.",
        ),
        default=[],
        value_type=RelationChoice(vocabulary="plone.app.vocabularies.Catalog"),
        required=False,
    )

    # custom widgets
    form.widget(
        "a_cura_di",
        RelatedItemsFieldWidget,
        vocabulary="plone.app.vocabularies.Catalog",
        pattern_options={
            "maximumSelectionSize": 1,
            "selectableTypes": ["UnitaOrganizzativa"],
        },
    )
    form.widget(
        "a_cura_di_persone",
        RelatedItemsFieldWidget,
        vocabulary="plone.app.vocabularies.Catalog",
        pattern_options={
            "selectableTypes": ["Persona"],
        },
    )
    form.widget(
        "notizie_correlate",
        RelatedItemsFieldWidget,
        vocabulary="plone.app.vocabularies.Catalog",
        pattern_options={
            "selectableTypes": ["News Item"],
        },
    )
    form.widget(
        "luoghi_correlati",
        RelatedItemsFieldWidget,
        vocabulary="plone.app.vocabularies.Catalog",
        pattern_options={
            "selectableTypes": ["Venue"],
        },
    )
    model.fieldset(
        "correlati",
        label=_("correlati_label", default="Contenuti collegati"),
        fields=["notizie_correlate"],
    )
    # custom fieldsets and order
    form.order_before(numero_progressivo_cs="ILeadImageBehavior.image")
    form.order_before(a_cura_di="ILeadImageBehavior.image")


@provider(IFormFieldProvider)
class INewsAdditionalFields(INewsAdditionalFieldsBase):
    descrizione_estesa = BlocksField(
        title=_("descrizione_estesa", default="Descrizione estesa"),
        required=True,
        description=_(
            "descrizione_estesa_help",
            default="Descrizione dettagliata e completa.",
        ),
    )

    form.order_before(descrizione_estesa="ILeadImageBehavior.image")
    textindexer.searchable("descrizione_estesa")


@implementer(INewsAdditionalFields)
@adapter(INewsItem)
class NewsAdditionalFields(object):
    """ """

    def __init__(self, context):
        self.context = context



# -*- coding: utf-8 -*-
from design.plone.contenttypes import _
from plone.app.registry.browser.controlpanel import ControlPanelFormWrapper
from plone.app.registry.browser.controlpanel import RegistryEditForm
from plone.z3cform import layout
from z3c.form import form
from zope.interface import Interface
from zope.schema import TextLine


class IGeolocationDefaults(Interface):
    street = TextLine(
        title=_("Via"),
        required=False,
        default="Via Liszt, 21",
    )
    zip_code = TextLine(
        title=_("CAP"),
        required=False,
        default="00144",
    )
    city = TextLine(
        title=_("Città"),
        required=False,
        default="Roma",
    )
    country = TextLine(
        title=_("Nazione"),
        required=False,
        default="{'title': 'Italia','token': '380',}",
    )
    geolocation = TextLine(
        title=_("Coordinate"),
        required=True,
        default="{'latitude': 41.8337833,'longitude': 12.4677863,}",
    )


class GeolocationDefaultControlPanelForm(RegistryEditForm):
    form.extends(RegistryEditForm)
    schema = IGeolocationDefaults
    id = "geolocation-defaults"
    label = _("Geolocation default")


GeolocationDefaultControlPanellView = layout.wrap_form(
    GeolocationDefaultControlPanelForm, ControlPanelFormWrapper
)
GeolocationDefaultControlPanellView.label = _("Geolocation default")



# -*- coding: utf-8 -*-
from collective.volto.blocksfield.field import BlocksField
from plone.app.dexterity import textindexer
from plone.app.event.base import default_timezone
from plone.app.z3cform.widget import (
    AjaxSelectFieldWidget,
    DatetimeFieldWidget,
    RelatedItemsFieldWidget,
)
from plone.autoform import directives
from plone.autoform import directives as form
from plone.supermodel import model
from redturtle.bandi import bandiMessageFactory as _
from redturtle.bandi import bandiMessageFactory as _rtbando
from redturtle.bandi.interfaces.bandoSchema import IBandoSchema, getDefaultEnte
from z3c.form.browser.checkbox import CheckBoxFieldWidget
from z3c.form.browser.radio import RadioFieldWidget
from z3c.relationfield.schema import RelationChoice, RelationList
from zope import schema

from design.plone.contenttypes.interfaces import IDesignPloneContentType


class IBandoAgidSchema(IBandoSchema, IDesignPloneContentType):
    """A Dexterity schema for Annoucements"""

    # ridefinito, così usiamo il campo dei blocchi
    cig = schema.TextLine(
        title=_("cig_label", default="CIG"),
        description=_("cig_help", default="Codice Identificativo di Gara (CIG)"),
        required=False,
    )
    text = BlocksField(
        title=_("text_label", default="Testo"),
        description=_(
            "text_help",
            default="",
        ),
        required=False,
    )
    tipologia_bando = schema.Choice(
        title=_rtbando("tipologia_bando_label", default="Announcement type"),
        description=_rtbando("tipologia_bando_help", default=""),
        vocabulary="redturtle.bandi.tipologia.vocabulary",
        required=True,
    )
    destinatari = schema.List(
        title=_rtbando("destinatari_label", default="Recipients"),
        description=_rtbando("destinatari_help", default=""),
        required=False,
        value_type=schema.Choice(vocabulary="redturtle.bandi.destinatari.vocabulary"),
    )
    ente_bando = schema.Tuple(
        title=_rtbando("ente_label", default="Authority"),
        description=_rtbando("ente_help", default="Select some authorities."),
        required=False,
        defaultFactory=getDefaultEnte,
        value_type=schema.TextLine(),
        missing_value=None,
    )
    apertura_bando = schema.Datetime(
        title=_("apertura_bando_label", default="Opening date"),
        description=_(
            "apertura_bando_help",
            default="Date and time of the opening of the announcement. Use "
            "this field if you want to set a specific opening date. "
            "If not set, the announcement will be open immediately.",
        ),
        required=False,
    )
    scadenza_domande_bando = schema.Datetime(
        title=_(
            "scadenza_domande_bando_label",
            default="Termine per le richieste di chiarimenti",
        ),
        description=_(
            "scadenza_domande_bando_help",
            default="Data entro la quale sarà possibile far pervenire domande"
            " e richieste di chiarimento a chi eroga il bando",
        ),
        required=False,
    )
    scadenza_bando = schema.Datetime(
        title=_rtbando("scadenza_bando_label", default="Expiration date and time"),
        description=_rtbando(
            "scadenza_bando_help",
            default="Deadline to participate in the announcement",
        ),
        required=False,
    )

    chiusura_procedimento_bando = schema.Date(
        title=_rtbando(
            "chiusura_procedimento_bando_label",
            default="Closing date procedure",
        ),
        description=_rtbando("chiusura_procedimento_bando_help", default=""),
        required=False,
    )

    riferimenti_bando = BlocksField(
        title=_("riferimenti_bando_agid_label", default="Ulteriori informazioni"),
        description=_(
            "riferimenti_bando_agid_help",
            default="Ulteriori informazioni non previste negli altri campi;"
            " si può trattare di contatti o note informative la cui conoscenza"
            " è indispensabile per la partecipazione al bando",
        ),
        required=False,
    )

    ufficio_responsabile = RelationList(
        title=_(
            "ufficio_responsabile_bando_label",
            default="Ufficio responsabile del bando",
        ),
        description=_(
            "ufficio_responsabile_bando_help",
            default="Seleziona l'ufficio responsabile di questo bando.",
        ),
        required=False,
        default=[],
        value_type=RelationChoice(
            title=_("Ufficio responsabile"),
            vocabulary="plone.app.vocabularies.Catalog",
        ),
    )

    area_responsabile = RelationList(
        title=_(
            "area_responsabile_label",
            default="Area responsabile del documento",
        ),
        description=_(
            "area_responsabile_help",
            default="Seleziona l'area amministrativa responsabile del " "documento.",
        ),
        required=False,
        default=[],
        value_type=RelationChoice(
            title=_("Area"), vocabulary="plone.app.vocabularies.Catalog"
        ),
    )

    # widgets
    directives.widget(
        "ente_bando",
        AjaxSelectFieldWidget,
        vocabulary="redturtle.bandi.enti.vocabulary",
    )
    directives.widget(destinatari=CheckBoxFieldWidget)
    directives.widget(tipologia_bando=RadioFieldWidget)
    form.widget(
        "ufficio_responsabile",
        RelatedItemsFieldWidget,
        vocabulary="plone.app.vocabularies.Catalog",
        pattern_options={
            "maximumSelectionSize": 1,
            "selectableTypes": ["UnitaOrganizzativa"],
        },
    )
    form.widget(
        "area_responsabile",
        RelatedItemsFieldWidget,
        vocabulary="plone.app.vocabularies.Catalog",
        pattern_options={
            "maximumSelectionSize": 1,
            "selectableTypes": ["UnitaOrganizzativa"],
        },
    )
    form.widget(
        "scadenza_domande_bando",
        DatetimeFieldWidget,
        default_timezone=default_timezone,
    )
    model.fieldset(
        "correlati",
        label=_("correlati_label", default="Contenuti collegati"),
        fields=["area_responsabile", "ufficio_responsabile"],
    )

    textindexer.searchable("text")
    textindexer.searchable("cig")



# -*- coding: utf-8 -*-
from design.plone.contenttypes.testing import (
    DESIGN_PLONE_CONTENTTYPES_INTEGRATION_TESTING,
    DESIGN_PLONE_CONTENTTYPES_API_FUNCTIONAL_TESTING,
)
from plone import api
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.app.testing import SITE_OWNER_NAME
from plone.app.testing import SITE_OWNER_PASSWORD
from plone.restapi.testing import RelativeSession
from redturtle.bandi.interfaces.settings import IBandoSettings

import unittest


class TestBandoSchema(unittest.TestCase):
    layer = DESIGN_PLONE_CONTENTTYPES_API_FUNCTIONAL_TESTING

    def setUp(self):
        self.app = self.layer["app"]
        self.portal = self.layer["portal"]
        self.request = self.layer["request"]
        self.portal_url = self.portal.absolute_url()
        setRoles(self.portal, TEST_USER_ID, ["Manager"])

        self.api_session = RelativeSession(self.portal_url)
        self.api_session.headers.update({"Accept": "application/json"})
        self.api_session.auth = (SITE_OWNER_NAME, SITE_OWNER_PASSWORD)

    def tearDown(self):
        self.api_session.close()

    def test_behaviors_enabled_for_bando(self):
        portal_types = api.portal.get_tool(name="portal_types")
        self.assertEqual(
            portal_types["Bando"].behaviors,
            (
                "plone.app.content.interfaces.INameFromTitle",
                "plone.app.dexterity.behaviors.discussion.IAllowDiscussion",
                "plone.app.dexterity.behaviors.exclfromnav.IExcludeFromNavigation",
                "plone.app.dexterity.behaviors.id.IShortName",
                "plone.app.dexterity.behaviors.metadata.IDublinCore",
                "plone.app.relationfield.behavior.IRelatedItems",
                "plone.app.versioningbehavior.behaviors.IVersionable",
                "plone.app.contenttypes.behaviors.tableofcontents.ITableOfContents",
                "plone.app.lockingbehavior.behaviors.ILocking",
                "Products.CMFPlone.interfaces.constrains.ISelectableConstrainTypes",
                "plone.versioning",
                "design.plone.contenttypes.behavior.argomenti_bando",
                "plone.textindexer",
                "plone.translatable",
                "kitconcept.seo",
                "design.plone.contenttypes.behavior.update_note",
                "volto.preview_image",
            ),
        )

    def test_bando_fieldsets(self):
        """
        Get the list from restapi
        """
        resp = self.api_session.get("@types/Bando").json()
        self.assertEqual(len(resp["fieldsets"]), 7)
        self.assertEqual(
            [x.get("id") for x in resp["fieldsets"]],
            [
                "default",
                "correlati",
                "settings",
                "categorization",
                "dates",
                "ownership",
                "seo",
            ],
        )

    def test_bando_required_fields(self):
        resp = self.api_session.get("@types/Bando").json()
        self.assertEqual(
            sorted(resp["required"]),
            sorted(["title", "tipologia_bando"]),
        )

    def test_bando_fields_default_fieldset(self):
        """
        Get the list from restapi
        """
        resp = self.api_session.get("@types/Bando").json()
        self.assertEqual(
            resp["fieldsets"][0]["fields"],
            [
                "title",
                "description",
                "cig",
                "text",
                "tipologia_bando",
                "destinatari",
                "ente_bando",
                "apertura_bando",
                "scadenza_domande_bando",
                "scadenza_bando",
                "chiusura_procedimento_bando",
                "riferimenti_bando",
                "update_note",
                "preview_image",
                "preview_caption",
            ],
        )

    def test_bando_fields_correlati_fieldset(self):
        """
        Get the list from restapi
        """
        resp = self.api_session.get("@types/Bando").json()
        self.assertEqual(
            resp["fieldsets"][1]["fields"],
            [
                "area_responsabile",
                "ufficio_responsabile",
                "tassonomia_argomenti",
                "correlato_in_evidenza",
            ],
        )

    def test_bando_fields_settings_fieldset(self):
        """
        Get the list from restapi
        """
        resp = self.api_session.get("@types/Bando").json()
        self.assertEqual(
            resp["fieldsets"][2]["fields"],
            [
                "allow_discussion",
                "exclude_from_nav",
                "id",
                "versioning_enabled",
                "table_of_contents",
                "changeNote",
            ],
        )

    def test_bando_fields_categorization_fieldset(self):
        """
        Get the list from restapi
        """
        resp = self.api_session.get("@types/Bando").json()
        self.assertEqual(
            resp["fieldsets"][3]["fields"],
            ["subjects", "language", "relatedItems"],
        )

    def test_bando_fields_dates_fieldset(self):
        """
        Get the list from restapi
        """
        resp = self.api_session.get("@types/Bando").json()
        self.assertEqual(resp["fieldsets"][4]["fields"], ["effective", "expires"])

    def test_bando_fields_ownership_fieldset(self):
        """
        Get the list from restapi
        """
        resp = self.api_session.get("@types/Bando").json()
        self.assertEqual(
            resp["fieldsets"][5]["fields"], ["creators", "contributors", "rights"]
        )

    def test_bando_fields_seo_fieldset(self):
        """
        Get the list from restapi
        """
        resp = self.api_session.get("@types/Bando").json()
        self.assertEqual(
            resp["fieldsets"][6]["fields"],
            [
                "seo_title",
                "seo_description",
                "seo_noindex",
                "seo_canonical_url",
                "opengraph_title",
                "opengraph_description",
                "opengraph_image",
            ],
        )


class TestBando(unittest.TestCase):
    layer = DESIGN_PLONE_CONTENTTYPES_INTEGRATION_TESTING

    def setUp(self):
        """Custom shared utility setup for tests."""
        self.portal = self.layer["portal"]
        self.request = self.layer["request"]
        self.portal_url = self.portal.absolute_url()
        setRoles(self.portal, TEST_USER_ID, ["Manager"])

    def test_bando_folder_deepening_addable_types(self):
        portal_types = api.portal.get_tool(name="portal_types")
        self.assertEqual(
            ("File", "Link", "Modulo"),
            portal_types["Bando Folder Deepening"].allowed_content_types,
        )

    def test_bando_view_base(self):
        portal_types = api.portal.get_tool(name="portal_types")
        self.assertEqual(portal_types["Bando"].default_view, "view")
        self.assertEqual(portal_types["Bando"].view_methods, ("view",))

    def test_disabled_default_ente(self):
        default_ente = api.portal.get_registry_record(
            "default_ente", interface=IBandoSettings
        )
        self.assertEqual(default_ente, ())

    def test_bando_substructure_created(self):
        bando = api.content.create(container=self.portal, type="Bando", title="Bando")

        self.assertIn("documenti", bando.keys())
        self.assertIn("comunicazioni", bando.keys())
        self.assertIn("esiti", bando.keys())



# -*- coding: utf-8 -*-
from .upgrades import colors
from .upgrades import logger
from .upgrades import update_catalog
from .upgrades import update_registry
from .upgrades import update_rolemap
from .upgrades import update_types
from Acquisition import aq_base
from collective.taxonomy.interfaces import ITaxonomy
from plone import api
from plone.base.utils import get_installer
from plone.namedfile.file import NamedBlobFile
from Products.CMFPlone.interfaces import ISelectableConstrainTypes
from Products.CMFPlone.utils import safe_hasattr
from zope.component import getUtilitiesFor


def to_7001(context):
    installer = get_installer(context=api.portal.get())
    installer.install_product("eea.api.taxonomy")
    logger.info(
        f"{colors.DARKCYAN} eea.api.taxonomy and collective.taxonomy installed {colors.ENDC}"  # noqa
    )
    # delete actual index from portal_catalog
    for index in [
        "tipologia_notizia",
        "tipologia_documento",
        "tipologia_organizzazione",
    ]:
        api.portal.get_tool("portal_catalog").delIndex(index)

    context.runImportStepFromProfile(
        "design.plone.contenttypes:taxonomy", "collective.taxonomy"
    )
    for utility_name, utility in list(getUtilitiesFor(ITaxonomy)):
        utility.updateBehavior(**{"field_prefix": ""})
        logger.info(
            f"{colors.DARKCYAN} Change taxonomy prefix for {utility_name} {colors.ENDC}"  # noqa
        )
    logger.info(
        f"{colors.DARKCYAN} design.plone.contentypes taxonomies imported {colors.ENDC}"  # noqa
    )
    update_types(context)
    update_registry(context)
    update_catalog(context)
    update_rolemap(context)
    logger.info(
        f"{colors.DARKCYAN} Upgraded types, registry, catalog and rolemap {colors.ENDC}"  # noqa
    )


def create_incarichi_folder(context):
    """TODO: documentare
    -> to 7001
    """
    logger.info(
        f"{colors.DARKCYAN} Inizio a creare la cartella Incarichi nelle persone {colors.ENDC}"  # noqa
    )
    pc = api.portal.get_tool(name="portal_catalog")
    wftool = api.portal.get_tool(name="portal_workflow")
    brains = pc({"portal_type": "Persona"})
    target = {"id": "incarichi", "title": "Incarichi", "contains": ("Incarico",)}
    for brain in brains:
        persona = brain.getObject()
        if target["id"] in persona:
            logger.info(
                f"{colors.YELLOW} {persona.title} contiene già la cartella incarichi {colors.ENDC}"  # noqa
            )
            continue
        suboject = api.content.create(
            type="Document", id=target["id"], title=target["title"], container=persona
        )
        subobjectConstraints = ISelectableConstrainTypes(suboject)
        subobjectConstraints.setConstrainTypesMode(1)
        subobjectConstraints.setLocallyAllowedTypes(target["contains"])

        if api.content.get_state(obj=persona) == "published":
            wftool.doActionFor(suboject, "publish")

        logger.info(
            f"{colors.GREEN} Creato la cartella incarichi per {persona.title}{colors.ENDC}"  # noqa
        )
    logger.info(
        f"{colors.DARKCYAN} Finito di creare la cartella Incarichi{colors.ENDC}"
    )


def create_incarico_for_persona(context):
    """TODO: documentare"""
    logger.info(
        f"{colors.DARKCYAN} Inizio a creare gli incarichi delle persone {colors.ENDC}"
    )
    # intids = getUtility(IIntIds)
    pc = api.portal.get_tool(name="portal_catalog")
    wftool = api.portal.get_tool(name="portal_workflow")
    brains = pc({"portal_type": "Persona"})
    MAPPING_TIPO = {
        "Amministrativa": "amministrativo",
        "Politica": "politico",
        "Altro tipo": "altro",
        "politica": "politico",  # parma
        "amministrativa": "amministrativo",  # parma
    }
    for brain in brains:
        persona = brain.getObject()

        incarichi_folder = persona["incarichi"]

        if incarichi_folder.values():
            logger.info(
                f"{colors.RED}{persona.title} ha già un incarico creato {colors.ENDC}"
            )  # noqa
            continue

        if safe_hasattr(persona, "ruolo"):
            # we could have the attribute and then remove the value
            incarico_title = persona.ruolo or f"Incarico di {persona.title}"
        else:
            logger.info(
                f"{colors.RED} Attenzione: {persona.title} non ha un ruolo {colors.ENDC}"  # noqa
            )
            incarico_title = f"Incarico di {persona.title}"

        incarico = api.content.create(
            type="Incarico", title=incarico_title, container=incarichi_folder
        )
        # incarico.persona = [RelationValue(intids.getId(persona))]
        api.relation.create(source=incarico, target=persona, relationship="persona")

        if safe_hasattr(persona, "organizzazione_riferimento"):
            incarico.unita_organizzativa = persona.organizzazione_riferimento

        if safe_hasattr(persona, "data_insediamento"):
            incarico.data_inizio_incarico = persona.data_insediamento
            incarico.data_insediamento = persona.data_insediamento

        if safe_hasattr(persona, "data_conclusione_incarico"):
            incarico.data_conclusione_incarico = persona.data_conclusione_incarico

        atto_nomina = None
        if safe_hasattr(persona, "atto_nomina") and getattr(persona, "atto_nomina"):
            atto_nomina = api.content.create(
                type="Documento",
                id="atto-di-nomina",
                title="Atto di nomina",
                container=incarico,
            )
            atto_nomina.description = f"Atto di nomina di {persona.title} per il ruolo di {incarico_title}"  # noqa
            # questo sotto rimane valido col vecchio schema del documento pubblico
            # atto_nomina.file_correlato = NamedBlobFile(
            #     data=persona.atto_nomina.data,
            #     filename=persona.atto_nomina.filename,
            #     contentType="application/pdf",
            # )
            # invece aggiungiamo un modulo sotto al documento
            modulo_atto_nomina = api.content.create(
                type="Modulo",
                id="atto-di-nomina",
                title="Atto di nomina",
                container=atto_nomina,
            )
            modulo_atto_nomina.file_principale = NamedBlobFile(
                data=persona.atto_nomina.data,
                filename=persona.atto_nomina.filename,
                contentType="application/pdf",
            )

            atto_nomina.tipologia_documento = ["documento_attivita_politica"]
            # incarico.atto_nomina = [RelationValue(intids.getId(atto_nomina))]
            api.relation.create(
                source=incarico, target=atto_nomina, relationship="atto_nomina"
            )
            logger.info(
                f"{colors.GREEN} Creato atto nomina per {persona.title} {colors.ENDC}"
            )

        if safe_hasattr(persona, "tipologia_persona") and persona.tipologia_persona:
            incarico.tipologia_incarico = MAPPING_TIPO[persona.tipologia_persona]

        # persona.incarichi_persona = [RelationValue(intids.getId(incarico))]
        api.relation.create(
            source=persona, target=incarico, relationship="incarichi_persona"
        )

        if api.content.get_state(obj=persona) == "published":
            wftool.doActionFor(incarico, "publish")
            wftool.doActionFor(incarico["compensi-file"], "publish")
            wftool.doActionFor(incarico["importi-di-viaggio-e-o-servizi"], "publish")
            if atto_nomina:
                wftool.doActionFor(atto_nomina, "publish")

        if safe_hasattr(persona, "compensi-file") and safe_hasattr(
            incarico, "compensi-file"
        ):
            compensi = getattr(persona, "compensi-file")
            for obj in compensi.contentItems():
                api.content.move(source=obj, target=incarico["compensi-file"])

        if safe_hasattr(persona, "importi-di-viaggio-e-o-servizi") and safe_hasattr(
            incarico, "importi-di-viaggio-e-o-servizi"
        ):
            importi = getattr(persona, "importi-di-viaggio-e-o-servizi")
            for obj in importi.contentItems():
                api.content.move(
                    source=obj, target=incarico["importi-di-viaggio-e-o-servizi"]
                )

        logger.info(f"{colors.GREEN} Creato incarico per {persona.title}{colors.ENDC}")

    logger.info(
        f"{colors.DARKCYAN} Finito di creare gli incarichi delle persone{colors.ENDC}"
    )


def create_pdc(context):
    """TODO: documentare (pdc = "punto di contatto")"""
    portal_types = ["UnitaOrganizzativa", "Persona", "Event", "Venue"]
    MAPPINGS = {
        "Persona": {
            "telefono": "telefono",
            "fax": "fax",
            "email": "email",
            "pec": "pec",
        },
        "UnitaOrganizzativa": {
            "telefono": "telefono",
            "fax": "fax",
            "email": "email",
            "pec": "pec",
            "web": "url",
        },
        "Event": {
            "telefono": "telefono",
            "fax": "fax",
            "email": "email",
            "web": "url",
        },
        "Venue": {
            "telefono": "telefono",
            "fax": "fax",
            "email": "email",
            "pec": "pec",
            "web": "url",
        },
    }

    def migrated_contact_info(source):
        # we check if we have attribute, if it's a list and no more a json (block field)
        # finally we check if we have at least a value.
        if (
            safe_hasattr(source, "contact_info")
            and type(obj.contact_info) == list  # noqa
            and len(obj.contact_info) > 0
        ):
            return True

    pc = api.portal.get_tool(name="portal_catalog")
    wftool = api.portal.get_tool(name="portal_workflow")
    portal = api.portal.get()
    punti_contatto_id = "punti-di-contatto"
    punti_contatto_title = "Punti di contatto"
    if "punti-di-contatto" not in portal:
        punti_contatto = api.content.create(
            type="Document",
            id=punti_contatto_id,
            title=punti_contatto_title,
            container=portal,
        )
        punti_contatto.exclude_from_nav = True
        punti_contatto.reindexObject()
        wftool.doActionFor(punti_contatto, "publish")
        logger.info(
            f"{colors.GREEN} Creato cartella punti di contatto nella radice del portal{colors.ENDC}"  # noqa
        )
    else:
        punti_contatto = portal[punti_contatto_id]

    for portal_type in portal_types:
        brains = pc(**{"portal_type": portal_type})
        logger.info(
            f"{colors.YELLOW} Stiamo per creare i PDC per {len(brains)} oggetti di tipo {portal_type}{colors.ENDC}"  # noqa
        )
        for brain in brains:
            obj = brain.getObject()
            mapping = MAPPINGS[portal_type]
            data = []
            for field in mapping:
                field_value = getattr(obj, field, None)
                if not field_value:
                    continue
                if type(field_value) != list:  # noqa
                    # in some case we have a f*****g list
                    field_value = [
                        field_value,
                    ]
                for value in field_value:
                    data.append({"pdc_type": mapping[field], "pdc_value": value})

            if not data:
                continue

            if not migrated_contact_info(obj):
                if hasattr(aq_base(obj), "contact_info"):
                    obj.old_contact_info = obj.contact_info
                    if obj.portal_type == "UnitaOrganizzativa":
                        del obj.contact_info
            else:
                logger.info(
                    f"{colors.RED} Esiste già un punto di contatto per {obj.title}({obj.absolute_url()}){colors.ENDC}"  # noqa
                )
                continue

            pdc = api.content.create(
                type="PuntoDiContatto",
                title=f"Punto di contatto per: {obj.title}",
                container=punti_contatto,
            )

            api.relation.create(source=obj, target=pdc, relationship="contact_info")

            pdc.value_punto_contatto = data
            # publish
            wftool.doActionFor(pdc, "publish")

            logger.info(
                f"{colors.GREEN} Creato il punto di contatto per {obj.title}({obj.absolute_url()}){colors.ENDC}"  # noqa
            )



# -*- coding: utf-8 -*-

from design.plone.contenttypes.testing import (
    DESIGN_PLONE_CONTENTTYPES_API_FUNCTIONAL_TESTING,
)
from plone import api
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.app.testing import SITE_OWNER_NAME
from plone.app.testing import SITE_OWNER_PASSWORD
from plone.restapi.testing import RelativeSession

import unittest


class TestDocumentSchema(unittest.TestCase):
    layer = DESIGN_PLONE_CONTENTTYPES_API_FUNCTIONAL_TESTING

    def setUp(self):
        self.app = self.layer["app"]
        self.portal = self.layer["portal"]
        self.request = self.layer["request"]
        self.portal_url = self.portal.absolute_url()
        setRoles(self.portal, TEST_USER_ID, ["Manager"])

        self.api_session = RelativeSession(self.portal_url)
        self.api_session.headers.update({"Accept": "application/json"})
        self.api_session.auth = (SITE_OWNER_NAME, SITE_OWNER_PASSWORD)

    def tearDown(self):
        self.api_session.close()

    def test_behaviors_enabled_for_document(self):
        portal_types = api.portal.get_tool(name="portal_types")
        self.assertEqual(
            portal_types["Document"].behaviors,
            (
                "plone.namefromtitle",
                "plone.allowdiscussion",
                "plone.excludefromnavigation",
                "plone.shortname",
                "plone.dublincore",
                "plone.relateditems",
                "plone.locking",
                "volto.blocks",
                "plone.versioning",
                "design.plone.contenttypes.behavior.info_testata",
                "design.plone.contenttypes.behavior.argomenti_document",
                "plone.translatable",
                "design.plone.contenttypes.behavior.show_modified",
                "kitconcept.seo",
                "plone.constraintypes",
                "design.plone.contenttypes.behavior.exclude_from_search",
                "plone.leadimage",
                "volto.preview_image",
            ),
        )

    def test_document_fieldsets(self):
        """
        Get the list from restapi
        """
        resp = self.api_session.get("@types/Document").json()
        self.assertEqual(len(resp["fieldsets"]), 9)
        self.assertEqual(
            [x.get("id") for x in resp["fieldsets"]],
            [
                "default",
                "testata",
                "settings",
                "correlati",
                "categorization",
                "dates",
                "ownership",
                "layout",
                "seo",
            ],
        )

    def test_document_required_fields(self):
        resp = self.api_session.get("@types/Document").json()
        self.assertEqual(
            sorted(resp["required"]),
            sorted(["title"]),
        )

    def test_document_fields_default_fieldset(self):
        """
        Get the list from restapi
        """
        resp = self.api_session.get("@types/Document").json()
        self.assertEqual(
            resp["fieldsets"][0]["fields"],
            [
                "title",
                "description",
            ],
        )

    def test_document_fields_testata_fieldset(self):
        """
        Get the list from restapi
        """
        resp = self.api_session.get("@types/Document").json()
        self.assertEqual(
            resp["fieldsets"][1]["fields"],
            [
                "image",
                "image_caption",
                "preview_image",
                "ricerca_in_testata",
                "mostra_bottoni_condivisione",
                "info_testata",
                "mostra_navigazione",
                "tassonomia_argomenti",
            ],
        )

    def test_document_fields_settings_fieldset(self):
        """
        Get the list from restapi
        """
        resp = self.api_session.get("@types/Document").json()
        self.assertEqual(
            resp["fieldsets"][2]["fields"],
            [
                "allow_discussion",
                "exclude_from_nav",
                "id",
                "versioning_enabled",
                "show_modified",
                "exclude_from_search",
                "changeNote",
            ],
        )

    def test_document_fields_correlati_fieldset(self):
        """
        Get the list from restapi
        """
        resp = self.api_session.get("@types/Document").json()
        self.assertEqual(
            resp["fieldsets"][3]["fields"],
            ["correlato_in_evidenza"],
        )

    def test_document_fields_categorization_fieldset(self):
        """
        Get the list from restapi
        """
        resp = self.api_session.get("@types/Document").json()
        self.assertEqual(
            resp["fieldsets"][4]["fields"], ["subjects", "language", "relatedItems"]
        )

    def test_document_fields_dates_fieldset(self):
        """
        Get the list from restapi
        """
        resp = self.api_session.get("@types/Document").json()
        self.assertEqual(resp["fieldsets"][5]["fields"], ["effective", "expires"])

    def test_document_fields_ownership_fieldset(self):
        """
        Get the list from restapi
        """
        resp = self.api_session.get("@types/Document").json()
        self.assertEqual(
            resp["fieldsets"][6]["fields"], ["creators", "contributors", "rights"]
        )

    def test_document_fields_layout_fieldset(self):
        """
        Get the list from restapi
        """
        resp = self.api_session.get("@types/Document").json()
        self.assertEqual(resp["fieldsets"][7]["fields"], ["blocks", "blocks_layout"])

    def test_document_fields_seo_fieldset(self):
        """
        Get the list from restapi
        """
        resp = self.api_session.get("@types/Document").json()
        self.assertEqual(
            resp["fieldsets"][8]["fields"],
            [
                "seo_title",
                "seo_description",
                "seo_noindex",
                "seo_canonical_url",
                "opengraph_title",
                "opengraph_description",
                "opengraph_image",
            ],
        )



# -*- coding: utf-8 -*-
from collective.z3cform.datagridfield.datagridfield import DataGridFieldFactory
from collective.z3cform.datagridfield.row import DictRow
from design.plone.contenttypes import _
from design.plone.contenttypes.interfaces import IDesignPloneContentType
from plone.app.z3cform.widget import RelatedItemsFieldWidget
from plone.autoform import directives as form
from plone.supermodel import model
from z3c.relationfield.schema import RelationChoice
from z3c.relationfield.schema import RelationList
from zope import schema


class IPDCValueSchema(model.Schema):
    pdc_type = schema.Choice(
        title=_("pdc_type_label", default="Tipo"),
        description=_(
            "type_help",
            default="Tipo",
        ),
        vocabulary="collective.taxonomy.tipologia_pdc",
        required=True,
        default="",
    )
    pdc_desc = schema.TextLine(
        title=_("pdc_desc_label", default="Descrizione"),
        description=_(
            "pdc_desc_help",
            default="Descrizione",
        ),
        required=False,
        default="",
        max_length=255,
    )
    pdc_value = schema.TextLine(
        title=_("pdc_value_label", default="Contatto"),
        description=_(
            "pdc_value_help",
            default="Contatto",
        ),
        required=True,
        default="",
        max_length=255,
    )


class IPuntoDiContatto(model.Schema, IDesignPloneContentType):
    """Marker interface for content type PuntoDiContatto"""

    value_punto_contatto = schema.List(
        title="Valore punto di contatto",
        default=[],
        value_type=DictRow(schema=IPDCValueSchema),
        description=_(
            "value_punto_contatto_help",
            default="Il valore del punto di contatto: il numero compreso di prefisso "
            "internazionale (se telefono), l'account (se social network), l'URL (se sito o pagina web), l'indirizzo email (se email).",  # noqa
        ),
        required=True,
    )
    persona = RelationList(
        title=_(
            "persona_incarico_label",
            default="Persona",
        ),
        description=_(
            "persona_incarico_help",
            default="Se una persona è un punto di contatto di un'altra Tipologia",
        ),
        value_type=RelationChoice(vocabulary="plone.app.vocabularies.Catalog"),
        required=False,
        default=[],
    )

    form.widget(
        "value_punto_contatto",
        DataGridFieldFactory,
        frontendOptions={
            "widget": "data_grid",
            "widgetProps": {
                "allow_reorder": True,
            },
        },
    )

    form.widget(
        "persona",
        RelatedItemsFieldWidget,
        vocabulary="plone.app.vocabularies.Catalog",
        pattern_options={
            "selectableTypes": ["Persona"],
        },
    )



# -*- coding: utf-8 -*-
from collective.volto.blocksfield.field import BlocksField
from design.plone.contenttypes import _
from design.plone.contenttypes.interfaces import IDesignPloneContentType
from plone.app.dexterity import textindexer
from plone.app.z3cform.widget import RelatedItemsFieldWidget
from plone.autoform import directives as form
from plone.namedfile import field
from plone.supermodel import model
from z3c.relationfield.schema import RelationChoice
from z3c.relationfield.schema import RelationList


# TODO: migration script for these commented fields towards PDC
# telefono
# fax
# email
# TODO: migration script for these commented fields towards Incarico
# atto_nomina
# data_conclusione_incarico
# data_insediamento


class IPersona(model.Schema, IDesignPloneContentType):
    """Marker interface for contenttype Persona"""

    foto_persona = field.NamedBlobImage(
        title=_("foto_persona_label", default="Foto della persona"),
        required=False,
        description=_(
            "foto_persona_help",
            default="Foto da mostrare della persona. "
            "La dimensione suggerita è 100x180px.",
        ),
    )
    # Questo campo per direttive e richieste viene nascosto nella form
    # Lo si tiene perche si vuole evitare di perder dati tra le migrazioni
    # e magari non poter piu' usare la feature collegata, ossia
    # la check persone, in quanto relazioni potrebbero rompersi o perdersi
    organizzazione_riferimento = RelationList(
        title=_(
            "organizzazione_riferimento_label",
            default="Organizzazione di riferimento",
        ),
        description=_(
            "organizzazione_riferimento_help",
            default="Seleziona una lista di organizzazioni a cui la persona"
            " appartiene.",
        ),
        value_type=RelationChoice(
            title=_("Organizzazione di riferimento"),
            vocabulary="plone.app.vocabularies.Catalog",
        ),
        default=[],
        required=False,
    )
    form.omitted("organizzazione_riferimento")
    incarichi_persona = RelationList(
        title=_(
            "incarichi_label",
            default="Incarichi",
        ),
        description=_(
            "incarichi_help",
            default="Seleziona l'incarico corrente della persona. Attenzione: è necessario selezionare sempre un incarico già creato e associarlo correttamente alla persona pubblica. Questa operazione è obbligatoria per garantire il superamento dell’asseverazione.",  # noqa
        ),
        value_type=RelationChoice(
            title=_("Incarichi"),
            vocabulary="plone.app.vocabularies.Catalog",
        ),
        default=[],
        required=False,
    )

    competenze = BlocksField(
        title=_("competenze_label", default="Competenze"),
        description=_(
            "competenze_help",
            default="Descrizione del ruolo e dei compiti della persona.",
        ),
        required=False,
    )
    deleghe = BlocksField(
        title=_("deleghe_label", default="Deleghe"),
        description=_(
            "deleghe_help",
            default="Elenco delle deleghe a capo della persona.",
        ),
        required=False,
    )

    biografia = BlocksField(
        title=_("biografia_label", default="Biografia"),
        description=_(
            "biografia_help",
            default="Solo per persona politica: testo descrittivo che riporta"
            " la biografia della persona.",
        ),
        required=False,
    )

    curriculum_vitae = field.NamedBlobFile(
        title=_("curriculum_vitae_label", default="Curriculum vitae"),
        required=False,
        description=_(
            "curriculum_vitae_help",
            default="Allega un file contenente il curriculum vitae della persona. "
            "Se ha più file da allegare, utilizza questo campo per quello principale "
            'e gli altri mettili dentro alla cartella "Curriculum vitae" che troverai dentro alla Persona.',  # noqa
        ),
    )

    emolumenti_a_carico_della_finanza_pubblica = field.NamedBlobFile(
        title=_(
            "emolumenti_a_carico_della_finanza_pubblica_label",
            default="Emolumenti a carico della finanza pubblica",
        ),
        required=False,
        description="",
    )

    dichiarazioni_di_insussistenza_e_incompatibilita = field.NamedBlobFile(
        title=_(
            "dichiarazioni_di_insussistenza_e_incompatibilita_label",
            default="Dichiarazioni di insussistenza e incompatibilità",
        ),
        required=False,
        description="",
    )

    # custom widgets
    form.widget(
        "organizzazione_riferimento",
        RelatedItemsFieldWidget,
        vocabulary="plone.app.vocabularies.Catalog",
        pattern_options={
            "selectableTypes": ["UnitaOrganizzativa"],
        },
    )

    form.widget(
        "incarichi_persona",
        RelatedItemsFieldWidget,
        vocabulary="plone.app.vocabularies.Catalog",
        pattern_options={
            "selectableTypes": ["Incarico"],
        },
    )

    # custom fieldsets
    model.fieldset(
        "ruolo",
        label=_("ruolo_label", default="Ruolo"),
        fields=[
            "incarichi_persona",
            "organizzazione_riferimento",
            "competenze",
            "deleghe",
            "biografia",
        ],
    )
    model.fieldset(
        "documenti",
        label=_("documenti_label", default="Documenti"),
        fields=[
            "curriculum_vitae",
            "emolumenti_a_carico_della_finanza_pubblica",
            "dichiarazioni_di_insussistenza_e_incompatibilita",
        ],
    )

    # SearchableText fields
    textindexer.searchable("competenze")
    textindexer.searchable("deleghe")
    # TODO: migration script for these commented fields towards PDC
    # textindexer.searchable("telefono")
    # textindexer.searchable("fax")
    # textindexer.searchable("email")



# -*- coding: utf-8 -*-
from design.plone.contenttypes.testing import (
    DESIGN_PLONE_CONTENTTYPES_API_FUNCTIONAL_TESTING,
)

from plone import api
from plone.app.testing import setRoles
from plone.app.testing import SITE_OWNER_NAME
from plone.app.testing import SITE_OWNER_PASSWORD
from plone.app.testing import TEST_USER_ID
from plone.app.testing.helpers import logout
from plone.indexer.interfaces import IIndexableObject
from plone.restapi.interfaces import IZCatalogCompatibleQuery
from plone.restapi.testing import RelativeSession
from transaction import commit
from zope.component import getMultiAdapter
from zope.component import queryMultiAdapter
import unittest


class ExcludeFromSearchFunctionalTest(unittest.TestCase):
    layer = DESIGN_PLONE_CONTENTTYPES_API_FUNCTIONAL_TESTING
    maxDiff = None

    def setUp(self):
        self.portal = self.layer["portal"]
        self.request = self.layer["request"]
        self.portal_url = self.portal.absolute_url()
        self.catalog = api.portal.get_tool("portal_catalog")
        setRoles(self.portal, TEST_USER_ID, ["Manager"])

        api.user.create(
            email="foo@example.com",
            username="foo",
            password="secret!!!",
        )

        self.news = api.content.create(
            container=self.portal,
            type="News Item",
            title="Test News",
        )

        self.document = api.content.create(
            container=self.portal,
            type="Document",
            title="Test Document",
        )

        api.content.transition(obj=self.news, transition="publish")
        api.content.transition(obj=self.news["multimedia"], transition="publish")
        api.content.transition(obj=self.document, transition="publish")

        commit()

        self.api_session = RelativeSession(self.portal_url)
        self.api_session.headers.update({"Accept": "application/json"})
        self.api_session.auth = (SITE_OWNER_NAME, SITE_OWNER_PASSWORD)

        self.api_session_foo = RelativeSession(self.portal_url)
        self.api_session_foo.headers.update({"Accept": "application/json"})
        self.api_session_foo.auth = ("foo", "secret!!!")

        self.api_session_anon = RelativeSession(self.portal_url)
        self.api_session_anon.headers.update({"Accept": "application/json"})

    def tearDown(self):
        self.api_session.close()
        self.api_session_anon.close()

    def test_exclude_from_search_indexer_for_item_without_behavior(self):
        """
        news item does not have the behavior, so it has False by default
        """
        self.assertRaises(AttributeError, getattr, self.news, "exclude_from_search")
        adapter = queryMultiAdapter((self.news, self.catalog), IIndexableObject)
        self.assertFalse(adapter.exclude_from_search)

    def test_exclude_from_search_indexer_for_item_with_behavior_enabled(self):
        """ """
        self.assertFalse(self.document.exclude_from_search)
        adapter = queryMultiAdapter((self.document, self.catalog), IIndexableObject)
        self.assertFalse(adapter.exclude_from_search)

    def test_exclude_from_search_indexer_for_item_with_behavior_enabled_and_set(self):
        """ """
        self.assertTrue(self.news["multimedia"].exclude_from_search)
        adapter = queryMultiAdapter(
            (self.news["multimedia"], self.catalog), IIndexableObject
        )
        self.assertTrue(adapter.exclude_from_search)

    def test_adapter_do_not_append_anything_to_query_for_auth_users(self):
        catalog_compatible_query = getMultiAdapter(
            (self.portal, self.request), IZCatalogCompatibleQuery
        )({})
        self.assertEqual({}, catalog_compatible_query)

    def test_adapter_append_exclude_from_search_to_query_for_anon_users(self):
        logout()
        catalog_compatible_query = getMultiAdapter(
            (self.portal, self.request), IZCatalogCompatibleQuery
        )({})
        self.assertEqual(catalog_compatible_query, {"exclude_from_search": False})

    def test_search_return_excluded_contents_for_logged_users(self):
        """ """
        resp = self.api_session.get(
            "/@search", params={"SearchableText": "multimedia"}
        ).json()
        self.assertEqual(resp["items_total"], 1)

        resp = self.api_session_foo.get(
            "/@search", params={"SearchableText": "multimedia"}
        ).json()
        self.assertEqual(resp["items_total"], 1)

    def test_search_do_not_return_excluded_contents_for_anon_users(self):
        """ """
        resp = self.api_session_anon.get(
            "/@search", params={"SearchableText": "multimedia"}
        ).json()
        self.assertEqual(resp["items_total"], 0)



# -*- coding: utf-8 -*-
from design.plone.contenttypes.interfaces.documento import IDocumento
from design.plone.contenttypes.utils import text_in_block
from plone.restapi.deserializer import json_body
from plone.restapi.deserializer.dxcontent import DeserializeFromJson
from plone.restapi.interfaces import IDeserializeFromJson
from zExceptions import BadRequest
from zope.component import adapter
from zope.interface import implementer
from zope.interface import Interface


TITLE_MAX_LEN = 255
DESCRIPTION_MAX_LEN = 255
EMPTY_BLOCK_MARKER = {"@type": "text"}
MANDATORY_RICH_TEXT_FIELDS = [
    # "descrizione_estesa",
    "formati_disponibili",
]


def new_error(message):
    return {"error": "ValidationError", "message": message}


@implementer(IDeserializeFromJson)
@adapter(IDocumento, Interface)
class DeserializeDocumentoFromJson(DeserializeFromJson):
    def __call__(
        self, validate_all=False, data=None, create=False
    ):  # noqa: ignore=C901
        if data is None:
            data = json_body(self.request)

        method = self.request.get("method")
        is_post = method == "POST"
        is_patch = method == "PATCH"
        errors = []

        if list(data.keys()) != ["ordering"]:
            title = data.get("title")
            description = data.get("description")
            if is_post:
                # Title validation
                if not title:
                    errors.append(new_error("Il titolo è obbligatorio"))
                elif len(title) > TITLE_MAX_LEN:
                    errors.append(
                        new_error(
                            "Il titolo può avere una lunghezza di massimo {} caratteri".format(  # noqa
                                TITLE_MAX_LEN
                            )
                        )
                    )

                # description validation
                if not description:
                    errors.append(new_error("La descrizione è obbligatoria"))
                elif len(description) > DESCRIPTION_MAX_LEN:
                    errors.append(
                        new_error(
                            "La descrizione deve avere una lunghezza di massimo {} caratteri".format(  # noqa
                                DESCRIPTION_MAX_LEN
                            )
                        )
                    )

                for field in MANDATORY_RICH_TEXT_FIELDS:
                    if field not in data:
                        errors.append(
                            new_error("Il campo {} è obbligatorio".format(field))
                        )
                    elif field in data and not text_in_block(data.get(field)):
                        errors.append(
                            new_error("Il campo {} è obbligatorio".format(field))
                        )

            if is_patch:
                # Title validation
                if "title" in data and not title:
                    errors.append(new_error("Il titolo è obbligatorio"))
                if title and len(title) > TITLE_MAX_LEN:
                    errors.append(
                        new_error(
                            "Il titolo può avere una lunghezza di massimo {} caratteri".format(  # noqa
                                TITLE_MAX_LEN
                            )
                        )
                    )
                # description validation
                if "description" in data and not description:
                    errors.append(new_error("La descrizione è obbligatoria"))
                if description and len(description) > DESCRIPTION_MAX_LEN:
                    errors.append(
                        new_error(
                            "La descrizione deve avere una lunghezza di massimo {} caratteri".format(  # noqa
                                DESCRIPTION_MAX_LEN
                            )
                        )
                    )
                if "description" not in data and not self.context.description:
                    errors.append(new_error("La descrizione è obbligatoria"))

                for field in MANDATORY_RICH_TEXT_FIELDS:
                    if field in data and not text_in_block(data.get(field)):
                        errors.append(
                            new_error("Il campo {} è obbligatorio".format(field))
                        )
                    # Se siamo nella patch siamo in modifica. Se siamo in modifica e
                    # siamo su un sito che ha avuto upgrade alla versione pnrr può
                    # essere che dei campi obbligatori un tempo non lo fossero e quindi
                    # arriviamo fino a qui
                    if field not in data and not text_in_block(
                        getattr(self.context, field)
                    ):
                        errors.append(
                            new_error("Il campo {} è obbligatorio".format(field))
                        )

        if errors:
            raise BadRequest(errors)
        return super(DeserializeDocumentoFromJson, self).__call__(
            validate_all=False, data=data, create=False
        )



from DateTime import DateTime
from openpyxl import Workbook
from openpyxl.styles import Alignment
from openpyxl.styles import Font
from openpyxl.styles import PatternFill
from openpyxl.utils import get_column_letter
from plone import api
from Products.Five import BrowserView

import io


FLAG = '<i class="fa-solid fa-check"></i>'


class CheckEventi(BrowserView):
    cds = None

    def is_anonymous(self):
        return api.user.is_anonymous()

    def information_dict(self, evento):
        prezzo = getattr(evento, "prezzo", "")
        res = [x.get("text", "") for x in prezzo["blocks"].values()]
        if not [x for x in res if x]:
            prezzo = ""

        luoghi_correlati = False
        if getattr(evento, "luoghi_correlati", None):
            luoghi_correlati = True
        elif getattr(evento, "geolocation", None):
            if getattr(evento.geolocation, "latitude", "") and getattr(
                evento.geolocation, "longitude", ""
            ):
                luoghi_correlati = True

        return {
            "description": getattr(evento, "description", "").strip(),
            "effective_date": getattr(evento, "effective_date", None),
            "luoghi_correlati": luoghi_correlati,
            "prezzo": prezzo,
            "contact_info": getattr(evento, "contact_info", None),
        }

    def plone2volto(self, url):
        portal_url = api.portal.get().absolute_url()
        frontend_domain = api.portal.get_registry_record(
            "volto.frontend_domain", default=""
        )
        if frontend_domain and url.startswith(portal_url):
            return url.replace(portal_url, frontend_domain, 1)
        return url

    def get_eventi(self):
        if self.is_anonymous():
            return []
        pc = api.portal.get_tool("portal_catalog")

        query = {
            "portal_type": "Event",
            "review_state": "published",
            "effectiveRange": DateTime(),
        }
        brains = pc(query)
        results = {}
        for brain in brains:
            evento = brain.getObject()

            information_dict = self.information_dict(evento)
            if all(information_dict.values()):
                continue

            parent = evento.aq_inner.aq_parent
            if parent.title not in results:
                results[parent.title] = {
                    "url": self.plone2volto(parent.absolute_url()),
                    "children": [],
                }
            results[parent.title]["children"].append(
                {
                    "title": evento.title,
                    "description": information_dict.get("description") and FLAG or "",
                    "url": self.plone2volto(evento.absolute_url()),
                    "data": {
                        "effective_date": information_dict.get("effective_date")
                        and FLAG
                        or "",
                        "luoghi_correlati": information_dict.get("luoghi_correlati")
                        and FLAG
                        or "",
                        "prezzo": information_dict.get("prezzo") and FLAG or "",
                        "contact_info": information_dict.get("contact_info")
                        and FLAG
                        or "",
                    },
                }
            )

        results = dict(sorted(results.items()))
        for key in results:
            results[key]["children"].sort(key=lambda x: x["title"])

        return results


class DownloadCheckEventi(CheckEventi):
    CT = "application/vnd.openxmlformats-officedocument.spreadsheetml.sheet"

    def __call__(self):
        HEADER = [
            "Titolo",
            "Descrizione",
            "Data",
            "Luogo",
            "Costo",
            "Contatti",
        ]

        EMPTY_ROW = [""] * 3

        eventi = self.get_eventi()

        workbook = Workbook()
        sheet = workbook.active
        sheet.title = "Check Persone"
        header_font = Font(bold=True)
        section_link_font = Font(underline="single", color="0563C1", size=14)
        link_fill = PatternFill(fill_type="solid", fgColor="DDDDDD")
        link_font = Font(underline="single", color="0563C1")
        section_fill = PatternFill(fill_type="solid", fgColor="E9E9E9")
        alignment = Alignment(horizontal="center", vertical="top", wrapText=True)

        section_row_height = int(14 * 1.5)

        for category, category_data in eventi.items():
            section_url = category_data["url"]
            section_title = category
            section_row = [section_title, "", ""]
            sheet.append(section_row)
            section_cell = sheet.cell(row=sheet.max_row, column=1)

            section_cell.alignment = alignment
            section_cell.hyperlink = section_url
            sheet.merge_cells(
                start_row=sheet.max_row,
                start_column=1,
                end_row=sheet.max_row,
                end_column=3,
            )  # noqa
            for row in sheet.iter_rows(
                min_row=sheet.max_row, max_row=sheet.max_row, min_col=1, max_col=3
            ):  # noqa
                sheet.row_dimensions[row[0].row].height = section_row_height
                for cell in row:
                    cell.fill = section_fill
                    cell.font = section_link_font

            sheet.append(HEADER)
            for col in range(1, len(HEADER) + 1):
                header_cell = sheet.cell(row=sheet.max_row, column=col)
                header_cell.fill = link_fill
                header_cell.font = header_font

            for col in sheet.columns:
                column_letter = get_column_letter(col[0].column)
                sheet.column_dimensions[column_letter].width = 35

            for evento in category_data["children"]:
                title_url = evento["url"]
                dati_evento = [
                    evento["title"],
                    "X" if evento["description"] else "",
                    "X" if evento["data"]["effective_date"] else "",
                    "X" if evento["data"]["luoghi_correlati"] else "",
                    "X" if evento["data"]["prezzo"] else "",
                    "X" if evento["data"]["contact_info"] else "",
                ]
                row = dati_evento
                sheet.append(row)

                title_cell = sheet.cell(row=sheet.max_row, column=1)
                check_cell = sheet.cell(row=sheet.max_row, column=2)
                check_cell.alignment = check_cell.alignment.copy(horizontal="center")
                title_cell.hyperlink = title_url
                title_cell.font = link_font
                column_letter_unit = get_column_letter(title_cell.column)
                sheet.column_dimensions[column_letter_unit].width = 60

            sheet.append(EMPTY_ROW)
            sheet.append(EMPTY_ROW)

        bytes_io = io.BytesIO()
        workbook.save(bytes_io)
        data = bytes_io.getvalue()
        self.request.response.setHeader("Content-Length", len(data))
        self.request.RESPONSE.setHeader("Content-Type", self.CT)
        self.request.response.setHeader(
            "Content-Disposition",
            "attachment; filename=check_eventi.xlsx",
        )
        return data



# -*- coding: utf-8 -*-
from design.plone.contenttypes.interfaces.pagina_argomento import IPaginaArgomento
from plone.app.dexterity.textindexer.interfaces import IDynamicTextIndexExtender
from zope.component import adapter
from zope.interface import implementer


HAVE_REST_API_PRE_961 = False

try:
    # plone 6.0.11 with last plone.restapi>9.6.0
    from plone.app.contenttypes.indexers import SearchableText
    from plone.restapi.indexers import get_blocks_text
    from plone.restapi.indexers import text_strip

except ImportError:
    # plone 6.0.10.1 with plone.restapi<9.6.1
    HAVE_REST_API_PRE_961 = True
    from plone.restapi.indexers import SearchableText_blocks


@adapter(IPaginaArgomento)
@implementer(IDynamicTextIndexExtender)
class SearchableTextExtender(object):
    def __init__(self, context):
        self.context = context

    def __call__(self):
        """Extend the searchable text with blocks"""
        if HAVE_REST_API_PRE_961:
            return SearchableText_blocks(self.context)()
        else:
            blocks_text = get_blocks_text(self.context)
            std_text = SearchableText(self.context)
            blocks_text.append(std_text)
            return text_strip(blocks_text)



# -*- coding: utf-8 -*-
from design.plone.contenttypes.interfaces import IDesignPloneContentType
from design.plone.contenttypes.schema_overrides import SchemaTweaks
from design.plone.contenttypes.testing import (
    DESIGN_PLONE_CONTENTTYPES_API_FUNCTIONAL_TESTING,
)
from plone import api
from plone.app.testing import setRoles
from plone.app.testing import SITE_OWNER_NAME
from plone.app.testing import SITE_OWNER_PASSWORD
from plone.app.testing import TEST_USER_ID
from plone.autoform.interfaces import IFormFieldProvider
from plone.restapi.testing import RelativeSession
from zope.component import provideAdapter

import transaction
import unittest


class TestEventSchema(unittest.TestCase):
    layer = DESIGN_PLONE_CONTENTTYPES_API_FUNCTIONAL_TESTING
    maxDiff = None

    def setUp(self):
        self.app = self.layer["app"]
        self.portal = self.layer["portal"]
        self.request = self.layer["request"]
        self.portal_url = self.portal.absolute_url()
        setRoles(self.portal, TEST_USER_ID, ["Manager"])

        self.api_session = RelativeSession(self.portal_url)
        self.api_session.headers.update({"Accept": "application/json"})
        self.api_session.auth = (SITE_OWNER_NAME, SITE_OWNER_PASSWORD)

    def tearDown(self):
        self.api_session.close()

    def test_behaviors_enabled_for_event(self):
        portal_types = api.portal.get_tool(name="portal_types")
        self.assertEqual(
            portal_types["Event"].behaviors,
            (
                "plone.eventbasic",
                "plone.leadimage",
                "volto.preview_image",
                "design.plone.contenttypes.behavior.argomenti_evento",
                "plone.eventrecurrence",
                "design.plone.contenttypes.behavior.additional_help_infos",
                "design.plone.contenttypes.behavior.evento",
                "design.plone.contenttypes.behavior.luoghi_correlati_evento",
                "design.plone.contenttypes.behavior.address_event",
                "design.plone.contenttypes.behavior.geolocation_event",
                "design.plone.contenttypes.behavior.strutture_correlate",
                "design.plone.contenttypes.behavior.contatti_event",
                "plone.dublincore",
                "plone.namefromtitle",
                "plone.allowdiscussion",
                "plone.excludefromnavigation",
                "plone.shortname",
                "plone.relateditems",
                "plone.versioning",
                "plone.locking",
                "plone.constraintypes",
                "plone.textindexer",
                "plone.translatable",
                "kitconcept.seo",
                "collective.taxonomy.generated.tipologia_evento",
            ),
        )

    def test_event_addable_types(self):
        portal_types = api.portal.get_tool(name="portal_types")
        self.assertEqual(
            sorted(("Image", "File", "Link", "Event", "Document")),
            sorted(portal_types["Event"].allowed_content_types),
        )

    def test_event_provide_design_pct_marker_interface(self):
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        event = api.content.create(container=self.portal, type="Event", title="Evento")
        self.assertTrue(IDesignPloneContentType.providedBy(event))

    def test_event_fieldsets(self):
        """
        Get the list from restapi
        """
        resp = self.api_session.get("@types/Event").json()
        self.assertEqual(len(resp["fieldsets"]), 12)
        # should be 13 but SchemaTweaks does not work in tests
        # self.assertEqual(len(resp["fieldsets"]), 13)
        self.assertEqual(
            [x.get("id") for x in resp["fieldsets"]],
            [
                "default",
                "cose",
                "luogo",
                "date_e_orari",
                "costi",
                "contatti",
                "informazioni",
                # "correlati", see SchemaTweaks problem in tests
                "categorization",
                "dates",
                "settings",
                "ownership",
                "seo",
            ],
        )

    def test_event_required_fields(self):
        resp = self.api_session.get("@types/Event").json()
        self.assertEqual(
            sorted(resp["required"]),
            sorted(
                [
                    "title",
                    "tassonomia_argomenti",
                    "tipologia_evento",
                    "start",
                    "prezzo",
                    "end",
                    "descrizione_estesa",
                    "descrizione_destinatari",
                    "contact_info",
                    "description",
                ]
            ),
        )

    def test_event_fields_default_fieldset(self):
        """
        Get the list from restapi
        """
        resp = self.api_session.get("@types/Event").json()
        self.assertEqual(
            resp["fieldsets"][0]["fields"],
            [
                "title",
                "description",
                "start",
                "end",
                "whole_day",
                "open_end",
                "sync_uid",
                "image",
                "image_caption",
                "preview_image",
                "preview_caption",
                "correlato_in_evidenza",
                "tassonomia_argomenti",
                "recurrence",
                "sottotitolo",
                "tipologia_evento",
            ],
            # should be like this with SchemaTweaks
            # [
            #     "title",
            #     "description",
            #     "image",
            #     "image_caption",
            #     "preview_image",
            #     "preview_caption",
            #     "correlato_in_evidenza",
            #     "tassonomia_argomenti",
            #     "sottotitolo",
            #     "tipologia_evento",
            # ],
        )

    def test_event_fields_cose_fieldset(self):
        """
        Get the list from restapi
        """
        resp = self.api_session.get("@types/Event").json()
        self.assertEqual(
            resp["fieldsets"][1]["fields"],
            [
                "descrizione_estesa",
                "descrizione_destinatari",
                "persone_amministrazione",
            ],
        )

    def test_event_fields_luogo_fieldset(self):
        """
        Get the list from restapi
        """
        resp = self.api_session.get("@types/Event").json()
        self.assertEqual(
            resp["fieldsets"][2]["fields"],
            [
                "luoghi_correlati",
                "nome_sede",
                "street",
                "zip_code",
                "city",
                "quartiere",
                "circoscrizione",
                "country",
                "geolocation",
            ],
        )

    def test_event_fields_date_e_orari_fieldset(self):
        """
        Get the list from restapi
        """
        resp = self.api_session.get("@types/Event").json()
        self.assertEqual(
            resp["fieldsets"][3]["fields"],
            [
                "orari",
            ],
            # with SchemaTweaks should be like this
            # [
            #     "start",
            #     "end",
            #     "whole_day",
            #     "open_end",
            #     "sync_uid",
            #     "recurrence",
            #     "orari",
            # ],
        )

    def test_event_fields_costi_fieldset(self):
        """
        Get the list from restapi
        """
        resp = self.api_session.get("@types/Event").json()
        self.assertEqual(
            resp["fieldsets"][4]["fields"],
            [
                "prezzo",
            ],
            # should be like this with SchemaTweaks
            # [
            #     "costi",
            # ],
        )

    def test_event_fields_contatti_fieldset(self):
        """
        Get the list from restapi
        """
        resp = self.api_session.get("@types/Event").json()
        self.assertEqual(
            resp["fieldsets"][5]["fields"],
            [
                "organizzato_da_interno",
                "organizzato_da_esterno",
                "supportato_da",
                "patrocinato_da",
                "contact_info",
            ],
        )

    def test_event_fields_informazioni_fieldset(self):
        """
        Get the list from restapi
        """
        resp = self.api_session.get("@types/Event").json()
        self.assertEqual(
            resp["fieldsets"][6]["fields"],
            ["ulteriori_informazioni", "strutture_politiche"],
        )

    # def test_event_fields_correlati_fieldset(self):
    #     """
    #     Get the list from restapi
    #     """
    #     resp = self.api_session.get("@types/Event").json()
    #     self.assertEqual(
    #         resp["fieldsets"][7]["fields"],
    #         ["relatedItems"],
    #     )

    def test_event_fields_categorization_fieldset(self):
        """
        Get the list from restapi
        """
        resp = self.api_session.get("@types/Event").json()
        self.assertEqual(
            resp["fieldsets"][7]["fields"],
            ["subjects", "language", "relatedItems"],
            # should be like this with SchemaTweaks
            # ["subjects", "language"],
        )

    def test_event_fields_dates_fieldset(self):
        """
        Get the list from restapi
        """
        resp = self.api_session.get("@types/Event").json()
        self.assertEqual(resp["fieldsets"][8]["fields"], ["effective", "expires"])

    def test_event_fields_settings_fieldset(self):
        """
        Get the list from restapi
        """
        resp = self.api_session.get("@types/Event").json()
        self.assertEqual(
            resp["fieldsets"][9]["fields"],
            [
                "allow_discussion",
                "exclude_from_nav",
                "id",
                "versioning_enabled",
                "changeNote",
            ],
        )

    def test_event_fields_ownership_fieldset(self):
        """
        Get the list from restapi
        """
        resp = self.api_session.get("@types/Event").json()
        self.assertEqual(
            resp["fieldsets"][10]["fields"], ["creators", "contributors", "rights"]
        )

    def test_event_fields_seo_fieldset(self):
        """
        Get the list from restapi
        """
        resp = self.api_session.get("@types/Event").json()
        self.assertEqual(
            resp["fieldsets"][11]["fields"],
            [
                "seo_title",
                "seo_description",
                "seo_noindex",
                "seo_canonical_url",
                "opengraph_title",
                "opengraph_description",
                "opengraph_image",
            ],
        )


class TestEventApi(unittest.TestCase):
    layer = DESIGN_PLONE_CONTENTTYPES_API_FUNCTIONAL_TESTING

    def setUp(self):
        self.app = self.layer["app"]
        self.portal = self.layer["portal"]
        self.portal_url = self.portal.absolute_url()
        setRoles(self.portal, TEST_USER_ID, ["Manager"])

        self.api_session = RelativeSession(self.portal_url)
        self.api_session.headers.update({"Accept": "application/json"})
        self.api_session.auth = (SITE_OWNER_NAME, SITE_OWNER_PASSWORD)
        self.event = api.content.create(
            container=self.portal, type="Event", title="Evento"
        )
        provideAdapter(SchemaTweaks, (IFormFieldProvider,), name="schema.tweaks")
        transaction.commit()

    def tearDown(self):
        self.api_session.close()

    def test_event_substructure_created(self):
        event = self.portal["evento"]

        self.assertEqual(
            sorted(["documenti", "immagini", "sponsor_evento", "video"]),
            sorted(event.keys()),
        )

        self.assertEqual(event["immagini"].portal_type, "Document")
        self.assertEqual(event["immagini"].constrain_types_mode, 1)
        self.assertEqual(event["immagini"].locally_allowed_types, ("Image", "Link"))

        self.assertEqual(event["sponsor_evento"].portal_type, "Document")
        self.assertEqual(event["sponsor_evento"].constrain_types_mode, 1)
        self.assertEqual(event["sponsor_evento"].locally_allowed_types, ("Link",))

        self.assertEqual(event["documenti"].portal_type, "Document")
        self.assertEqual(event["documenti"].constrain_types_mode, 1)
        self.assertEqual(event["documenti"].locally_allowed_types, ("File",))

        self.assertEqual(event["video"].portal_type, "Document")
        self.assertEqual(event["video"].constrain_types_mode, 1)
        self.assertEqual(event["video"].locally_allowed_types, ("Link",))

        self.assertEqual(api.content.get_state(obj=event["immagini"]), "published")
        self.assertEqual(api.content.get_state(obj=event["video"]), "published")
        self.assertEqual(
            api.content.get_state(obj=event["sponsor_evento"]), "published"
        )
        self.assertEqual(api.content.get_state(obj=event["documenti"]), "published")



# -*- coding: utf-8 -*-
import logging

import transaction
from plone import api
from plone.app.upgrade.utils import installOrReinstallProduct
from Products.CMFPlone.interfaces import ISelectableConstrainTypes

from design.plone.contenttypes.events.common import SUBFOLDERS_MAPPING, createStructure
from design.plone.contenttypes.utils import create_default_blocks

logger = logging.getLogger(__name__)

DEFAULT_PROFILE = "profile-design.plone.contenttypes:default"


def update_profile(context, profile, run_dependencies=True):
    context.runImportStepFromProfile(DEFAULT_PROFILE, profile, run_dependencies)


def update_catalog(context):
    update_profile(context, "catalog")


def update_registry(context):
    update_profile(context, "plone.app.registry", run_dependencies=False)


def to_7301(context):
    brains = api.content.find(portal_type="Persona")
    for brain in brains:
        persona = brain.getObject()
        FOLDER_ID = "altri-documenti"
        if FOLDER_ID not in persona.keys():
            child = api.content.create(
                container=persona,
                type="Document",
                title="Altri documenti",
                id=FOLDER_ID,
            )
            create_default_blocks(context=child)
        else:
            child = persona[FOLDER_ID]

        child.exclude_from_search = True
        child.reindexObject(idxs=["exclude_from_search"])
        # select constraints
        constraintsChild = ISelectableConstrainTypes(child)
        constraintsChild.setConstrainTypesMode(1)
        constraintsChild.setLocallyAllowedTypes(
            (
                "File",
                "Documento",
                "Link",
            )
        )
        if api.content.get_state(persona) == "published":
            if api.content.get_state(child) != "published":
                with api.env.adopt_roles(["Reviewer"]):
                    api.content.transition(obj=child, transition="publish")


def to_7302(context):
    update_catalog(context)
    brains = api.content.find(portal_type="Event")
    tot = len(brains)
    i = 0
    for brain in brains:
        i += 1
        if i % 100 == 0:
            logger.info("Progress: {}/{}".format(i, tot))
        event = brain.getObject()
        event.reindexObject(idxs=["rassegna"])


def to_7303(context):
    update_registry(context)
    logger.info("Update registry")


def to_7304(context):
    brains = context.portal_catalog(**{"portal_type": "Event"})
    tot = len(brains)
    i = 0
    for brain in brains:
        i += 1
        if i % 100 == 0:
            logger.info("Progress: {}/{}".format(i, tot))
            transaction.commit()
        doc = brain.getObject()
        doc.reindexObject(idxs=["parent"])


def to_7305(context):
    mapping = SUBFOLDERS_MAPPING["Servizio"]
    mapping = [
        folder for folder in mapping.get("content") if folder["id"] == "modulistica"
    ][
        0
    ]  # noqa
    brains = api.content.find(portal_type="Servizio")
    tot = len(brains)
    i = 0
    for brain in brains:
        i += 1
        if i % 100 == 0:
            logger.info("Progress: {}/{}".format(i, tot))
            transaction.commit()
        servizio = brain.getObject()
        FOLDER_ID = "modulistica"
        if FOLDER_ID not in servizio.keys():
            continue
        child = servizio[FOLDER_ID]
        constraintsChild = ISelectableConstrainTypes(child)
        constraintsChild.setConstrainTypesMode(1)
        constraintsChild.setLocallyAllowedTypes(mapping["allowed_types"])


def to_7306(context):
    logger.info("Enable plone.constraintypes behavior and filter types")

    portal_types = api.portal.get_tool(name="portal_types")
    behavior = "plone.constraintypes"
    for ct in ["Persona", "Incarico"]:
        portal_type = portal_types.get(ct, None)
        if portal_type:
            portal_type.filter_content_types = True
            behaviors = list(portal_type.behaviors)
            if behavior not in behaviors:
                behaviors.append(behavior)
                portal_type.behaviors = tuple(behaviors)


def to_7307(context):
    logger.info("Update registry")
    update_registry(context)
    logger.info("Add new effectivestart (DateRecurringIndex) index")

    class extra:
        recurdef = "recurrence"
        until = ""

    name = "effectivestart"
    catalog = api.portal.get_tool(name="portal_catalog")
    catalog.addIndex(name, "DateRecurringIndex", extra=extra())
    logger.info("Catalog DateRecurringIndex {} created.".format(name))


def to_7308(context):
    logger.info("Reindex Events")
    pc = api.portal.get_tool(name="portal_catalog")
    brains = pc(portal_type="Event")
    tot = len(brains)
    for i, brain in enumerate(brains):
        if i % 15 == 0:
            logger.info("Progress: {}/{}".format(i, tot))
        brain.getObject().reindexObject(idxs=["effectivestart"])


def to_7309(context):
    logger.info("Uninstall eea.api.taxonomy")
    ps = api.portal.get_tool(name="portal_setup")
    ps.runAllImportStepsFromProfile(
        "profile-design.plone.contenttypes:remove_eea_api_taxonomy"
    )
    ps.unsetLastVersionForProfile("eea.api.taxonomy:default")
    logger.info("Install blocksfield")
    installOrReinstallProduct(api.portal.get(), "collective.volto.blocksfield")


def to_7310(context):
    logger.info("Enable kitconcept.seo behavior to File")
    portal_types = api.portal.get_tool(name="portal_types")
    behavior = "kitconcept.seo"
    for ct in ["File"]:
        portal_type = portal_types.get(ct, None)
        if portal_type:
            behaviors = list(portal_type.behaviors)
            if behavior not in behaviors:
                behaviors.append(behavior)
                portal_type.behaviors = tuple(behaviors)


def to_7311(context):
    allowed = list(context.portal_types["Documento"].allowed_content_types)
    if "File" not in allowed:
        allowed.append("File")
        context.portal_types["Documento"].allowed_content_types = tuple(allowed)
    logger.info("Update ct documento addables")


def to_7312(context):
    """
    Only add metadata if missing. We don't want to force reindexing.
    """
    pc = api.portal.get_tool(name="portal_catalog")
    for column in ["image_caption", "preview_caption"]:
        if column not in pc.schema():
            pc.addColumn(column)


def to_7313(context):
    logger.info("Update registry")
    context.runImportStepFromProfile(
        "profile-design.plone.contenttypes:to_7313", "plone.app.registry", False
    )

    logger.info("Add new effectiveend (DateRecurringIndex) index")

    class extra:
        recurdef = "recurrence"
        until = ""

    name = "effectiveend"
    catalog = api.portal.get_tool(name="portal_catalog")

    if "effectiveend" not in catalog.indexes():
        catalog.addIndex(name, "DateRecurringIndex", extra=extra())
        logger.info("Catalog DateRecurringIndex {} created.".format(name))

    logger.info("Reindex Events")
    brains = catalog(portal_type="Event")
    tot = len(brains)
    for i, brain in enumerate(brains):
        if i % 15 == 0:
            logger.info("Progress: {}/{}".format(i, tot))
        brain.getObject().reindexObject(idxs=["effectiveend"])


def to_7314(context):
    logger.info("Add new folder to Persona CT")
    mapping1 = {
        "content": [
            {
                "id": "dichiarazione-insussistenza-cause-di-inconferibilita-e-incompatibilita",  # noqa
                "title": "Dichiarazione insussistenza cause di inconferibilità e"
                " incompatibilità",
                "allowed_types": ("File",),
            },
        ]
    }
    mapping2 = {
        "content": [
            {
                "id": "emolumenti-complessivi-percepiti-a-carico-della-finanza-pubblica",  # noqa
                "title": "Emolumenti complessivi percepiti a carico della finanza"
                " pubblica",
                "allowed_types": ("File",),
            },
        ]
    }
    pc = api.portal.get_tool(name="portal_catalog")
    brains = pc(portal_type="Persona")
    for brain in brains:
        persona = brain.getObject()
        actual_contraints = ISelectableConstrainTypes(persona)
        actual_constrain_type = actual_contraints.getConstrainTypesMode()
        # devo poter aggiungere
        actual_contraints.setConstrainTypesMode(0)
        if (
            "dichiarazione-insussistenza-cause-di-inconferibilita-e-incompatibilita"
            not in persona.keys()
        ):
            createStructure(persona, mapping1)
            logger.info("Add dichiarazione insussistenza for {}".format(persona.title))
        if (
            "emolumenti-complessivi-percepiti-a-carico-della-finanza-pubblica"
            not in persona.keys()
        ):
            createStructure(persona, mapping2)
            logger.info("Add emolumenti complessivi for {}".format(persona.title))
        # reimposto il valore precedente del constraintypes
        actual_contraints.setConstrainTypesMode(actual_constrain_type)


def to_7315(context):
    wanted = (("cig", "FieldIndex"),)
    pc = api.portal.get_tool(name="portal_catalog")
    for name, meta_type in wanted:
        if name not in pc.indexes():
            pc.addIndex(name, meta_type)
            pc.addColumn(name)
            logger.info("Added index {} of type {}".format(name, meta_type))
        else:
            logger.info("Index {} already exists".format(name))
    logger.info("Reindex Bandi")



# -*- coding: utf-8 -*-

"""Setup tests for this package."""
from design.plone.contenttypes.testing import (
    DESIGN_PLONE_CONTENTTYPES_API_FUNCTIONAL_TESTING,
)
from plone import api
from plone.app.testing import setRoles
from plone.app.testing import SITE_OWNER_NAME
from plone.app.testing import SITE_OWNER_PASSWORD
from plone.app.testing import TEST_USER_ID
from plone.restapi.testing import RelativeSession
from Products.CMFPlone.utils import getToolByName
from transaction import commit
from z3c.relationfield import RelationValue
from zope.component import getUtility
from zope.intid.interfaces import IIntIds
from zope.schema.interfaces import IVocabularyFactory

import transaction
import unittest


class TestUOSchema(unittest.TestCase):
    layer = DESIGN_PLONE_CONTENTTYPES_API_FUNCTIONAL_TESTING

    def setUp(self):
        self.app = self.layer["app"]
        self.portal = self.layer["portal"]
        self.request = self.layer["request"]
        self.portal_url = self.portal.absolute_url()
        setRoles(self.portal, TEST_USER_ID, ["Manager"])

        self.api_session = RelativeSession(self.portal_url)
        self.api_session.headers.update({"Accept": "application/json"})
        self.api_session.auth = (SITE_OWNER_NAME, SITE_OWNER_PASSWORD)

    def tearDown(self):
        self.api_session.close()

    def test_behaviors_enabled_for_uo(self):
        portal_types = api.portal.get_tool(name="portal_types")
        self.assertEqual(
            portal_types["UnitaOrganizzativa"].behaviors,
            (
                "plone.namefromtitle",
                "plone.allowdiscussion",
                "plone.excludefromnavigation",
                "plone.shortname",
                "plone.ownership",
                "plone.publication",
                "plone.categorization",
                "plone.basic",
                "plone.locking",
                "plone.leadimage",
                "volto.preview_image",
                "plone.relateditems",
                "design.plone.contenttypes.behavior.contatti_uo",
                "design.plone.contenttypes.behavior.argomenti",
                "plone.textindexer",
                "design.plone.contenttypes.behavior.additional_help_infos",
                "plone.translatable",
                "kitconcept.seo",
                "plone.versioning",
                "collective.taxonomy.generated.tipologia_organizzazione",
            ),
        )

    def test_uo_ct_title(self):
        portal_types = api.portal.get_tool(name="portal_types")
        self.assertEqual(
            "Unita Organizzativa", portal_types["UnitaOrganizzativa"].title
        )

    def test_uo_fieldsets(self):
        """
        Get the list from restapi
        """
        resp = self.api_session.get("@types/UnitaOrganizzativa").json()
        self.assertEqual(len(resp["fieldsets"]), 12)
        self.assertEqual(
            [x.get("id") for x in resp["fieldsets"]],
            [
                "default",
                "cosa_fa",
                "struttura",
                "persone",
                "contatti",
                "correlati",
                "categorization",
                "informazioni",
                "settings",
                "ownership",
                "dates",
                "seo",
            ],
        )

    def test_uo_required_fields(self):
        resp = self.api_session.get("@types/UnitaOrganizzativa").json()
        self.assertEqual(
            sorted(resp["required"]),
            sorted(
                [
                    "title",
                    "competenze",
                    "tipologia_organizzazione",
                    "sede",
                    "contact_info",
                    "description",
                ]
            ),
        )

    def test_uo_fields_default_fieldset(self):
        """
        Get the list from restapi
        """
        resp = self.api_session.get("@types/UnitaOrganizzativa").json()
        self.assertEqual(
            resp["fieldsets"][0]["fields"],
            [
                "title",
                "description",
                "image",
                "image_caption",
                "preview_image",
                "preview_caption",
                "tassonomia_argomenti",
            ],
        )

    def test_uo_fields_cosa_fa_fieldset(self):
        """
        Get the list from restapi
        """
        resp = self.api_session.get("@types/UnitaOrganizzativa").json()
        self.assertEqual(resp["fieldsets"][1]["fields"], ["competenze"])

    def test_uo_fields_struttura_fieldset(self):
        """
        Get the list from restapi
        """
        resp = self.api_session.get("@types/UnitaOrganizzativa").json()
        self.assertEqual(
            resp["fieldsets"][2]["fields"],
            [
                "legami_con_altre_strutture",
                "responsabile",
                "assessore_riferimento",
                "tipologia_organizzazione",
            ],
        )

    def test_uo_fields_persone_fieldset(self):
        """
        Get the list from restapi
        """
        resp = self.api_session.get("@types/UnitaOrganizzativa").json()
        self.assertEqual(
            resp["fieldsets"][3]["fields"],
            ["persone_struttura"],
        )

    def test_uo_fields_contatti_fieldset(self):
        """
        Get the list from restapi
        """
        resp = self.api_session.get("@types/UnitaOrganizzativa").json()
        self.assertEqual(
            resp["fieldsets"][4]["fields"],
            ["contact_info", "sede", "sedi_secondarie", "orario_pubblico"],
        )

    def test_uo_fields_correlati_fieldset(self):
        """
        Get the list from restapi
        """
        resp = self.api_session.get("@types/UnitaOrganizzativa").json()
        self.assertEqual(
            resp["fieldsets"][5]["fields"],
            # ["documenti_pubblici", "correlato_in_evidenza"], # BBB dovrebbe essere così
            # ma viene fuori così nei test perché non viene vista la patch SchemaTweaks
            ["correlato_in_evidenza"],
        )

    def test_uo_fields_categorization_fieldset(self):
        """
        Get the list from restapi
        """
        resp = self.api_session.get("@types/UnitaOrganizzativa").json()
        self.assertEqual(
            resp["fieldsets"][6]["fields"],
            # ["subjects", "language"] BBB dovrebbe essere così
            # ma nei test esce così perché non viene vista la patch di SchemaTweaks
            ["subjects", "language", "relatedItems", "documenti_pubblici"],
        )

    def test_uo_fields_informazioni_fieldset(self):
        """
        Get the list from restapi
        """
        resp = self.api_session.get("@types/UnitaOrganizzativa").json()
        self.assertEqual(resp["fieldsets"][7]["fields"], ["ulteriori_informazioni"])

    def test_uo_fields_settings_fieldset(self):
        """
        Get the list from restapi
        """
        resp = self.api_session.get("@types/UnitaOrganizzativa").json()
        self.assertEqual(
            resp["fieldsets"][8]["fields"],
            [
                "allow_discussion",
                "exclude_from_nav",
                "id",
                "versioning_enabled",
                "changeNote",
            ],
        )

    def test_uo_fields_ownership_fieldset(self):
        """
        Get the list from restapi
        """
        resp = self.api_session.get("@types/UnitaOrganizzativa").json()
        self.assertEqual(
            resp["fieldsets"][9]["fields"], ["creators", "contributors", "rights"]
        )

    def test_uo_fields_dates_fieldset(self):
        """
        Get the list from restapi
        """
        resp = self.api_session.get("@types/UnitaOrganizzativa").json()
        self.assertEqual(resp["fieldsets"][10]["fields"], ["effective", "expires"])

    def test_uo_fields_seo_fieldset(self):
        """
        Get the list from restapi
        """
        resp = self.api_session.get("@types/UnitaOrganizzativa").json()
        self.assertEqual(
            resp["fieldsets"][11]["fields"],
            [
                "seo_title",
                "seo_description",
                "seo_noindex",
                "seo_canonical_url",
                "opengraph_title",
                "opengraph_description",
                "opengraph_image",
            ],
        )


class TestUO(unittest.TestCase):
    """Test that design.plone.contenttypes is properly installed."""

    layer = DESIGN_PLONE_CONTENTTYPES_API_FUNCTIONAL_TESTING
    maxDiff = None

    def setUp(self):
        self.app = self.layer["app"]
        self.portal = self.layer["portal"]
        self.portal_url = self.portal.absolute_url()
        setRoles(self.portal, TEST_USER_ID, ["Manager"])

        self.api_session = RelativeSession(self.portal_url)
        self.api_session.headers.update({"Accept": "application/json"})
        self.api_session.auth = (SITE_OWNER_NAME, SITE_OWNER_PASSWORD)

        self.intids = getUtility(IIntIds)
        self.news = api.content.create(
            container=self.portal, type="News Item", title="TestNews"
        )
        self.luogo = api.content.create(
            container=self.portal,
            type="Venue",
            title="Luogo",
        )
        sede = RelationValue(self.intids.getId(self.luogo))

        self.uo = api.content.create(
            container=self.portal,
            type="UnitaOrganizzativa",
            title="TestUO",
            sede=[sede],
            city="Metropolis",
            zip_code="1234",
            street="whatever",
            contact_info={"blocks": {"xxx": {"foo": "bar"}}},
        )

        self.uo_child = api.content.create(
            container=self.uo,
            type="UnitaOrganizzativa",
            title="Child uo",
            city="Gotham City",
            zip_code="5678",
            street="somewhere",
            contact_info={"blocks": {"yyy": {"foo": "bar"}}},
        )

        self.service = api.content.create(
            container=self.portal,
            type="Servizio",
            title="TestService",
            ufficio_responsabile=[RelationValue(self.intids.getId(self.uo))],
        )

        self.bando = api.content.create(
            container=self.portal,
            type="Bando",
            title="TestBando",
            ufficio_responsabile=[RelationValue(self.intids.getId(self.uo))],
        )

        self.news.a_cura_di = [RelationValue(self.intids.getId(self.uo))]
        pcatalog = getToolByName(self.portal, "portal_catalog")
        pcatalog.manage_reindexIndex(ids=["ufficio_responsabile", "news_uo"])
        commit()

    def tearDown(self):
        self.api_session.close()

    def test_uo_service_related_service_show_only_services(self):
        response = self.api_session.get(self.uo.absolute_url() + "?fullobjects")
        self.assertEqual(
            len(response.json()["servizi_offerti"]),
            1,
        )
        self.assertTrue(
            response.json()["servizi_offerti"][0]["@id"],
            self.service.absolute_url(),
        )

    def test_uo_sede_data(self):
        response = self.api_session.get(self.uo.absolute_url() + "?fullobjects")
        sede = response.json()["sede"][0]
        fields = [
            "street",
            "zip_code",
            "city",
            "country",
            "orario_pubblico",
            "telefono",
            "email",
            "pec",
            "web",
            "riferimento_telefonico_struttura",
            "riferimento_mail_struttura",
            "riferimento_pec_struttura",
        ]
        for field in fields:
            self.assertEqual(sede[field], getattr(self.luogo, field, None))

    def test_uo_location_indexer_populated(self):
        pc = api.portal.get_tool(name="portal_catalog")
        self.assertEqual((self.luogo.UID(),), pc.uniqueValuesFor("uo_location"))

        # create another uo and a venue
        luogo_bis = api.content.create(
            container=self.portal,
            type="Venue",
            title="Luogo bis",
        )
        api.content.create(
            container=self.portal,
            type="UnitaOrganizzativa",
            title="Second UO",
            sede=[RelationValue(self.intids.getId(self.luogo))],
            sedi_secondarie=[RelationValue(self.intids.getId(luogo_bis))],
        )

        self.assertEqual(
            tuple(sorted([self.luogo.UID(), luogo_bis.UID()])),
            pc.uniqueValuesFor("uo_location"),
        )

    def test_uo_locations_vocabulary_populated(self):
        # create another uo and a venue
        luogo_bis = api.content.create(
            container=self.portal,
            type="Venue",
            title="Luogo bis",
        )
        api.content.create(
            container=self.portal,
            type="UnitaOrganizzativa",
            title="Second UO",
            sede=[RelationValue(self.intids.getId(self.luogo))],
            sedi_secondarie=[RelationValue(self.intids.getId(luogo_bis))],
        )

        factory = getUtility(
            IVocabularyFactory, "design.plone.vocabularies.uo_locations"
        )
        vocabulary = factory(self.portal)

        self.assertIn(self.luogo.UID(), vocabulary)
        self.assertIn(luogo_bis.UID(), vocabulary)
        self.assertEqual(len(vocabulary), 2)
        self.assertEqual(vocabulary.getTerm(self.luogo.UID()).title, self.luogo.Title())
        self.assertEqual(vocabulary.getTerm(luogo_bis.UID()).title, luogo_bis.Title())

    def test_do_not_show_parent_uo_if_not_present(self):
        response = self.api_session.get(self.uo.absolute_url())
        self.assertEqual(response.status_code, 200)
        uo_parent = response.json()["uo_parent"]

        self.assertIsNone(uo_parent)

    def test_show_parent_uo_if_present(self):
        response = self.api_session.get(self.uo_child.absolute_url())
        self.assertEqual(response.status_code, 200)
        uo_parent = response.json()["uo_parent"]

        self.assertIsNotNone(uo_parent)
        self.assertEqual(uo_parent["id"], self.uo.getId())
        self.assertEqual(uo_parent["zip_code"], self.uo.zip_code)
        self.assertEqual(uo_parent["city"], self.uo.city)
        self.assertEqual(uo_parent["contact_info"], self.uo.contact_info)
        self.assertEqual(uo_parent["street"], self.uo.street)

    def test_do_not_show_children_uo_if_not_present(self):
        response = self.api_session.get(self.uo_child.absolute_url())
        self.assertEqual(response.status_code, 200)
        uo_children = response.json()["uo_children"]

        self.assertEqual(uo_children, [])

    def test_show_children_uo_if_present(self):
        response = self.api_session.get(self.uo.absolute_url())
        self.assertEqual(response.status_code, 200)
        uo_children = response.json()["uo_children"]

        self.assertEqual(len(uo_children), 1)
        self.assertEqual(uo_children[0]["id"], self.uo_child.getId())
        self.assertEqual(uo_children[0]["zip_code"], self.uo_child.zip_code)
        self.assertEqual(uo_children[0]["city"], self.uo_child.city)
        self.assertEqual(uo_children[0]["contact_info"], self.uo_child.contact_info)
        self.assertEqual(uo_children[0]["street"], self.uo_child.street)

    def test_backref_to_servizio_dove_rivolgersi(self):
        """Check backref to servizio in
        dove rivolgersi field"""

        self.service.dove_rivolgersi = [RelationValue(self.intids.getId(self.uo))]

        self.service.reindexObject()
        transaction.commit()

        response = self.api_session.get(self.uo.absolute_url())
        self.assertEqual(response.status_code, 200)
        self.assertIn(
            self.service.absolute_url(),
            [i["@id"] for i in response.json()["prestazioni"]],
        )

    def test_cant_patch_uo_that_has_no_required_fields(self):
        uo = api.content.create(
            container=self.portal, type="UnitaOrganizzativa", title="Foo"
        )
        commit()
        resp = self.api_session.patch(
            uo.absolute_url(),
            json={
                "title": "Foo modified",
            },
        )
        self.assertEqual(resp.status_code, 400)
        self.assertIn("La descrizione è obbligatoria", resp.json()["message"])

    def test_can_sort_uo_that_has_no_required_fields(self):
        uo = api.content.create(
            container=self.portal, type="UnitaOrganizzativa", title="Foo"
        )
        commit()
        self.assertEqual(self.bando, self.portal.listFolderContents()[-2])
        self.assertEqual(uo, self.portal.listFolderContents()[-1])

        resp = self.api_session.patch(
            self.portal_url,
            json={"ordering": {"delta": -1, "obj_id": uo.getId()}},
        )
        commit()

        self.assertEqual(resp.status_code, 204)
        self.assertEqual(self.bando, self.portal.listFolderContents()[-1])
        self.assertEqual(uo, self.portal.listFolderContents()[-2])



from openpyxl import Workbook
from openpyxl.styles import Font
from plone import api
from plone.namedfile.browser import Download
from Products.Five import BrowserView
from zope.interface import implementer
from zope.interface import Interface
from zope.publisher.interfaces import IPublishTraverse
from zope.publisher.interfaces import NotFound

import io


class IUsersSummaryView(Interface):
    """marker interface"""


@implementer(IUsersSummaryView)
class UsersSummaryView(BrowserView):
    """"""

    def __call__(self):
        return

    def absolute_url(self):
        """
        Needed for plone.namedfile >= 6.4.0 with canonical header
        """
        return f"{self.context.absolute_url()}/{self.__name__}"


@implementer(IPublishTraverse)
class UsersSummaryDownload(Download):
    def __init__(self, context, request):
        super().__init__(context=context, request=request)
        self.data = None

    def publishTraverse(self, request, name):
        super().publishTraverse(request=request, name=name)

        if name == "users.xlsx":
            # XXX: al momento l'estrazione avviene a partire dai gruppi,
            #      questo perchè il sito potrebbe essere connesso ad un LDAP
            #      aziendale contenente molti utenti e non ci interessa estrarli
            #      tutti

            users = {}
            groups = [
                g.getId()
                for g in api.group.get_groups()
                if g.getId() not in ["AuthenticatedUsers"]
            ]
            for group in api.group.get_groups():
                for userid in group.getGroupMemberIds():
                    if userid not in users:
                        user = api.user.get(userid=userid)
                        if user:
                            users[userid] = {
                                "username": user.getUserName(),
                                "email": user.getProperty("email"),
                                "fullname": user.getProperty("fullname"),
                            }
                            for g in user.getGroups():
                                users[userid][g] = "x"
            workbook = Workbook()
            sheet = workbook.active
            sheet.title = f"Redazione {api.portal.get().Title()}"
            header_font = Font(bold=True)
            sheet.append(
                [
                    "USERNAME",
                    "EMAIL",
                    "NOME",
                ]
            )
            sheet.cell(row=1, column=1).font = header_font
            sheet.cell(row=1, column=2).font = header_font
            sheet.cell(row=1, column=3).font = header_font
            for idx, g in enumerate(groups):
                sheet.cell(row=1, column=4 + idx).value = g
                sheet.cell(row=1, column=4 + idx).font = header_font

            for row, user in enumerate(
                sorted(users.values(), key=lambda u: u["username"])
            ):
                sheet.append(
                    [
                        user["username"],
                        user["email"],
                        user["fullname"],
                    ]
                )
                for col, g in enumerate(groups):
                    sheet.cell(row=row + 2, column=4 + col).value = user.get(g)

            bytes_io = io.BytesIO()
            workbook.save(bytes_io)
            self.data = bytes_io.getvalue()
            self.request.response.setHeader("Content-Length", len(self.data))
            self.request.RESPONSE.setHeader(
                "Content-Type",
                "application/vnd.openxmlformats-officedocument.spreadsheetml.sheet",
            )
            self.request.response.setHeader(
                "Content-Disposition",
                "attachment; filename=users.xlsx",
            )
        else:
            self.data = None

        return self

    # def _getFile(self):
    def __call__(self):
        if not self.data:
            raise NotFound(self, "b", self.request)
        return self.data



# -*- coding: utf-8 -*-
from collective.address.behaviors import IAddress
from design.plone.contenttypes import _
from plone.app.dexterity import textindexer
from plone.autoform.interfaces import IFormFieldProvider
from plone.dexterity.interfaces import IDexterityContent
from plone.supermodel import model
from zope import schema
from zope.component import adapter
from zope.interface import implementer
from zope.interface import provider


class IAddressNomeSede(model.Schema):
    nome_sede = schema.TextLine(
        title=_("nome_sede", default="Nome sede"),
        description=_(
            "help_nome_sede",
            default="Inserisci il nome della "
            "sede, se non è presente tra i Luoghi del sito.",
        ),
        required=False,
    )
    textindexer.searchable("nome_sede")


class IAddressLocal(model.Schema):
    """ """

    quartiere = schema.TextLine(
        title=_("quartiere", default="Quartiere"),
        description=_("help_quartiere", default=""),
        required=False,
    )

    circoscrizione = schema.TextLine(
        title=_("circoscrizione", default="Circoscrizione"),
        description=_("help_circoscrizione", default=""),
        required=False,
    )

    # searchabletext indexer
    textindexer.searchable("quartiere")
    textindexer.searchable("circoscrizione")


@provider(IFormFieldProvider)
class IAddressVenue(IAddress, IAddressLocal):
    """"""

    model.fieldset(
        "dove",
        label=_("dove_label", default="Dove"),
        fields=[
            "street",
            "zip_code",
            "city",
            "quartiere",
            "circoscrizione",
            "country",
        ],
    )


@provider(IFormFieldProvider)
class IAddressEvent(IAddress, IAddressNomeSede, IAddressLocal):
    """"""

    model.fieldset(
        "luogo",
        label=_("luogo_label", default="Luogo"),
        fields=[
            "nome_sede",
            "street",
            "zip_code",
            "city",
            "quartiere",
            "circoscrizione",
            "country",
        ],
    )


@implementer(IAddressVenue)
@adapter(IDexterityContent)
class AddressVenue(object):
    """ """

    def __init__(self, context):
        self.context = context


@implementer(IAddressEvent)
@adapter(IDexterityContent)
class AddressEvent(object):
    """ """

    def __init__(self, context):
        self.context = context



# -*- coding: utf-8 -*-
from collective.volto.blocksfield.field import BlocksField
from design.plone.contenttypes import _
from design.plone.contenttypes.interfaces import IDesignPloneContentType
from plone.app.z3cform.widget import RelatedItemsFieldWidget
from plone.autoform import directives as form
from plone.supermodel import model
from z3c.relationfield.schema import RelationChoice
from z3c.relationfield.schema import RelationList
from zope import schema


class IIncarico(model.Schema, IDesignPloneContentType):
    """Marker interface for content type Incarico"""

    compensi = BlocksField(
        title=_(
            "compensi_incarico_label",
            default="Compensi",
        ),
        description=_(
            "compensi_incarico_help",
            default="Solo per incarico politico: compensi di qualsiasi natura"
            " connessi all'assunzione della carica.",
        ),
        required=False,
    )

    importi_viaggio_servizio = BlocksField(
        title=_(
            "importi_viaggio_servizio_incarico_label",
            default="Importi di viaggio e/o servizio",
        ),
        description=_(
            "importi_viaggio_servizio_incarico_help",
            default="Solo per incarico politico: importi di viaggi di servizio"
            "  e missioni pagati con fondi pubblici.",
        ),
        required=False,
    )

    persona = RelationList(
        title=_(
            "persona_incarico_label",
            default="La persona che ha la carica e l'incarico",
        ),
        description=_(
            "persona_incarico_help",
            default="Seleziona la persona che ha questo incarico",
        ),
        required=False,
        default=[],
        value_type=RelationChoice(
            title=_("Persona"),
            vocabulary="plone.app.vocabularies.Catalog",
        ),
    )

    unita_organizzativa = RelationList(
        title=_(
            "unita_organizzativa_incarico_label",
            default="Unità organizzativa",
        ),
        description=_(
            "unita_organizzativa_incarico_help",
            default="Seleziona l'organizzazione presso la quale svolge l'incarico.",
        ),
        required=False,
        default=[],
        value_type=RelationChoice(
            title=_("Unità organizzativa"),
            vocabulary="plone.app.vocabularies.Catalog",
        ),
    )

    responsabile_struttura = RelationList(
        title=_(
            "responsabile_struttura_incarico_label",
            default="Responsabile della struttura",
        ),
        description=_(
            "responsabile_struttura_incarico_help",
            default="Se è un incarico di responsabilità, specificare l'organizzazione "
            "della quale è responsabile in base all'incarico",
        ),
        required=False,
        default=[],
        value_type=RelationChoice(
            title=_("Responsabile della struttura"),
            vocabulary="plone.app.vocabularies.Catalog",
        ),
    )

    data_inizio_incarico = schema.Date(
        title=_("data_inizio_incarico", default="Data inizio incarico"),
        required=False,
    )

    data_conclusione_incarico = schema.Date(
        title=_("data_conclusione_incarico", default="Data conclusione incarico"),
        required=False,
    )

    data_insediamento = schema.Date(
        title=_("data_insediamento", default="Data insediamento"),
        required=False,
    )

    atto_nomina = RelationList(
        title=_(
            "atto_nomina_incarico_label",
            default="Atto di nomina",
        ),
        description=_(
            "atto_nomina_incarico_help",
            default="Inserire riferimento all'atto di nomina della persona",
        ),
        required=False,
        default=[],
        value_type=RelationChoice(
            title=_("Atto di nomina"),
            vocabulary="plone.app.vocabularies.Catalog",
        ),
    )

    # custom widgets
    form.widget(
        "unita_organizzativa",
        RelatedItemsFieldWidget,
        vocabulary="plone.app.vocabularies.Catalog",
        pattern_options={
            "maximumSelectionSize": 1,
            "selectableTypes": ["UnitaOrganizzativa"],
        },
    )

    form.widget(
        "persona",
        RelatedItemsFieldWidget,
        vocabulary="plone.app.vocabularies.Catalog",
        pattern_options={
            "maximumSelectionSize": 1,
            "selectableTypes": ["Persona"],
        },
    )

    form.widget(
        "responsabile_struttura",
        RelatedItemsFieldWidget,
        vocabulary="plone.app.vocabularies.Catalog",
        pattern_options={
            "maximumSelectionSize": 1,
            "selectableTypes": ["UnitaOrganizzativa"],
        },
    )

    form.widget(
        "atto_nomina",
        RelatedItemsFieldWidget,
        vocabulary="plone.app.vocabularies.Catalog",
        pattern_options={
            "maximumSelectionSize": 1,
            "selectableTypes": ["Documento"],
        },
    )

    #  custom fieldsets
    model.fieldset(
        "informazioni_compensi",
        label=_("informazioni_compensi_label", default="Compensi e trasparenza"),
        fields=["compensi", "importi_viaggio_servizio"],
    )
    model.fieldset(
        "date_e_informazioni",
        label=_("date_e_informazioni_label", default="Date e informazioni"),
        fields=["data_conclusione_incarico", "data_insediamento", "atto_nomina"],
    )



# -*- coding: utf-8 -*-
from Products.Five.browser import BrowserView


class TrasparenzaView(BrowserView):
    """ """

    def __init__(self, context, request):
        self.context = context
        self.request = request

    def __call__(self):
        return


class DettagliProcedimentiView(BrowserView):
    """ """

    def __init__(self, context, request):
        self.context = context
        self.request = request

    def __call__(self):
        return



# -*- coding: utf-8 -*-
from plone.app.contenttypes.interfaces import IEvent
from plone.indexer.decorator import indexer
from plone.event.interfaces import IEventAccessor
from plone.event.interfaces import IRecurrenceSupport


@indexer(IEvent)
def event_location(context, **kw):
    """ """
    luoghi_correlati = [x.to_object for x in context.luoghi_correlati]
    luoghi_correlati = filter(bool, luoghi_correlati)
    luoghi_correlati_title = [x.UID() for x in luoghi_correlati]
    return luoghi_correlati_title


@indexer(IEvent)
def rassegna(context, **kw):
    """ """
    children = context.values()
    return "Event" in [child.portal_type for child in children]


@indexer(IEvent)
def effectivestart(obj):
    start = IEventAccessor(obj).start
    if not start:
        raise AttributeError
    return start


@indexer(IEvent)
def effectiveend(obj):
    occurrences = IRecurrenceSupport(obj).occurrences()
    end = obj.end
    for occurrence in list(occurrences):
        if occurrence.end > end:
            end = occurrence.end
    return end



# -*- coding: utf-8 -*-
from AccessControl import getSecurityManager
from design.plone.contenttypes.interfaces.persona import IPersona
from plone.restapi.deserializer import json_body
from plone.restapi.deserializer.dxcontent import DeserializeFromJson
from plone.restapi.interfaces import IDeserializeFromJson
from zope.component import adapter
from zope.interface import implementer
from zope.interface import Interface


@implementer(IDeserializeFromJson)
@adapter(IPersona, Interface)
class DeserializePersonaFromJson(DeserializeFromJson):
    def __init__(self, context, request):
        self.context = context
        self.request = request

        self.sm = getSecurityManager()
        self.permission_cache = {}
        self.modified = {}

    def __call__(
        self, validate_all=False, data=None, create=False
    ):  # noqa: ignore=C901
        if data is None:
            data = json_body(self.request)
        if data:
            if (
                "data_insediamento" in data
                and data["data_insediamento"]
                and len(data["data_insediamento"]) > 10
            ):
                data["data_insediamento"] = data["data_insediamento"][0:10]

            if (
                "data_conclusione_incarico" in data
                and data["data_conclusione_incarico"]
                and len(data["data_conclusione_incarico"]) > 10
            ):
                data["data_conclusione_incarico"] = data["data_conclusione_incarico"][
                    0:10
                ]
        return super(DeserializePersonaFromJson, self).__call__(
            validate_all=False, data=data, create=False
        )



from design.plone.contenttypes.interfaces import IDesignPloneContenttypesLayer
from plone.restapi.interfaces import IZCatalogCompatibleQuery
from plone.restapi.search.query import ZCatalogCompatibleQueryAdapter as BaseAdapter
from zope.component import adapter
from zope.interface import implementer
from zope.interface import Interface
from plone import api


@implementer(IZCatalogCompatibleQuery)
@adapter(Interface, IDesignPloneContenttypesLayer)
class ZCatalogCompatibleQueryAdapter(BaseAdapter):
    """ """

    def __call__(self, query):
        """
        Do not show excluded from search items when anonymous are performing
        some catalog searches
        """
        query = super().__call__(query=query)

        if api.user.is_anonymous():
            # For the anonymous user, only content that is not "excluded from the search" is found
            query["exclude_from_search"] = False

        return query



# -*- coding: utf-8 -*-
from design.plone.contenttypes.interfaces.persona import IPersona
from plone.dexterity.content import Container
from zope.interface import implementer


@implementer(IPersona)
class Persona(Container):
    """ """



# -*- coding: utf-8 -*-
from design.plone.contenttypes.interfaces import IDesignPloneContentType
from plone.app.dexterity.textindexer.converters import (
    DefaultDexterityTextIndexFieldConverter,
)
from plone.app.dexterity.textindexer.interfaces import IDexterityTextIndexFieldConverter
from plone.dexterity.interfaces import IDexterityContent
from plone.restapi.indexers import (
    TextBlockSearchableText as BaseTextBlockSearchableText,
)
from plone.restapi.interfaces import IBlockSearchableText
from z3c.form.interfaces import IWidget
from z3c.relationfield.interfaces import IRelationChoice
from z3c.relationfield.interfaces import IRelationList
from zope.component import adapter
from zope.interface import implementer
from zope.publisher.interfaces.browser import IBrowserRequest


@implementer(IDexterityTextIndexFieldConverter)
@adapter(IDexterityContent, IRelationChoice, IWidget)
class RelationChoiceFieldConverter(DefaultDexterityTextIndexFieldConverter):
    def convert(self):
        relation = self.field.get(self.context)
        if not relation:
            return ""
        related_obj = relation.to_object
        if not related_obj:
            return ""
        return related_obj.Title()


@implementer(IDexterityTextIndexFieldConverter)
@adapter(IDexterityContent, IRelationList, IWidget)
class RelationListFieldConverter(DefaultDexterityTextIndexFieldConverter):
    def convert(self):
        relations = self.field.get(self.context)
        if not relations:
            return ""
        return " ".join([x.to_object.Title() for x in relations if x.to_object])


@implementer(IBlockSearchableText)
@adapter(IDesignPloneContentType, IBrowserRequest)
class TextBlockSearchableText(BaseTextBlockSearchableText):
    """ """



# -*- coding: utf-8 -*-
"""Setup tests for this package."""
from design.plone.contenttypes.controlpanels.settings import IDesignPloneSettings
from design.plone.contenttypes.testing import (
    DESIGN_PLONE_CONTENTTYPES_INTEGRATION_TESTING,
)
from plone import api
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID

import unittest


try:
    from Products.CMFPlone.utils import get_installer
except ImportError:
    get_installer = None


class TestSetup(unittest.TestCase):
    """Test that design.plone.contenttypes is properly installed."""

    layer = DESIGN_PLONE_CONTENTTYPES_INTEGRATION_TESTING

    def setUp(self):
        """Custom shared utility setup for tests."""
        self.portal = self.layer["portal"]
        if get_installer:
            self.installer = get_installer(self.portal, self.layer["request"])
        else:
            self.installer = api.portal.get_tool("portal_quickinstaller")

    def test_product_installed(self):
        """Test if design.plone.contenttypes is installed."""
        if hasattr(self.installer, "is_product_installed"):
            # Plone 6
            self.assertTrue(
                self.installer.is_product_installed("design.plone.contenttypes")
            )
        else:
            # Plone 5
            self.assertTrue(
                self.installer.isProductInstalled("design.plone.contenttypes")
            )

    def test_browserlayer(self):
        """Test that IDesignPloneContenttypesLayer is registered."""
        from design.plone.contenttypes.interfaces import IDesignPloneContenttypesLayer
        from plone.browserlayer import utils

        self.assertIn(IDesignPloneContenttypesLayer, utils.registered_layers())

    def test_show_modified_default_enabled(self):
        value = api.portal.get_registry_record(
            "show_modified_default", interface=IDesignPloneSettings
        )
        self.assertTrue(value)


class TestUninstall(unittest.TestCase):
    layer = DESIGN_PLONE_CONTENTTYPES_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer["portal"]
        if get_installer:
            self.installer = get_installer(self.portal, self.layer["request"])
        else:
            self.installer = api.portal.get_tool("portal_quickinstaller")
        roles_before = api.user.get_roles(TEST_USER_ID)
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        if hasattr(self.installer, "uninstall_product"):
            # Plone 6
            self.installer.uninstall_product("design.plone.contenttypes")
        else:
            # Plone 5
            self.installer.uninstallProducts(["design.plone.contenttypes"])
        setRoles(self.portal, TEST_USER_ID, roles_before)

    def test_product_uninstalled(self):
        """Test if design.plone.contenttypes is cleanly uninstalled."""
        if hasattr(self.installer, "is_product_installed"):
            # Plone 6
            self.assertFalse(
                self.installer.is_product_installed("design.plone.contenttypes")
            )
        else:
            # Plone 5
            self.assertFalse(
                self.installer.isProductInstalled("design.plone.contenttypes")
            )

    def test_browserlayer_removed(self):
        """Test that IDesignPloneContenttypesLayer is removed."""
        from design.plone.contenttypes.interfaces import IDesignPloneContenttypesLayer
        from plone.browserlayer import utils

        self.assertNotIn(IDesignPloneContenttypesLayer, utils.registered_layers())



# -*- coding: utf-8 -*-
from collective.volto.blocksfield.field import BlocksField
from design.plone.contenttypes import _
from plone.app.dexterity import textindexer
from plone.app.z3cform.widget import RelatedItemsFieldWidget
from plone.autoform import directives as form
from plone.autoform.interfaces import IFormFieldProvider
from plone.dexterity.interfaces import IDexterityContent
from plone.supermodel import model
from z3c.relationfield.schema import RelationChoice
from z3c.relationfield.schema import RelationList
from zope import schema
from zope.component import adapter
from zope.interface import implementer
from zope.interface import provider


@provider(IFormFieldProvider)
class ILuogo(model.Schema):
    """ """

    # moved to behavior under field name descrizione_estesa?
    descrizione_completa = BlocksField(
        title=_("descrizione_completa", default="Descrizione completa"),
        description=_(
            "help_descrizione_completa",
            default="Indicare una descrizione completa, inserendo tutte le"
            " informazioni rilevanti relative al luogo",
        ),
        required=False,
    )

    nome_alternativo = schema.TextLine(
        title=_("nome_alternativo", default="Nome alternativo"),
        description=_(
            "help_nome_alternativo",
            default="Indicare, se esiste, un nome alternativo per il luogo;"
            " questo sarà mostrato affianco al titolo della"
            " scheda",
        ),
        required=False,
    )

    elementi_di_interesse = BlocksField(
        title=_("elementi_di_interesse", default="Elementi di interesse"),
        description=_(
            "help_elementi_di_interesse",
            default="Indicare eventuali elementi di interesse per il " "cittadino.",
        ),
        required=False,
    )

    modalita_accesso = BlocksField(
        title=_("modalita_accesso", default="Modalita' di accesso"),
        description=_(
            "help_modalita_accesso",
            default="Indicare tutte le informazioni relative alla modalità di"
            " accesso al luogo",
        ),
        required=True,
    )

    struttura_responsabile_correlati = RelationList(
        title=_(
            "struttura_responsabile_correlati",
            default="Struttura responsabile del luogo.",
        ),
        description=_(
            "struttura_responsabile_correlati_help",
            default="Indicare la struttura responsabile del luogo qualora sia"
            " fra unità organizzative del comune inserite nel sito; altrimenti"
            " compilare i campi testuali relativi alla struttura responsabile",
        ),
        value_type=RelationChoice(
            title=_("Struttura responsabile"),
            vocabulary="plone.app.vocabularies.Catalog",
        ),
        required=False,
        default=[],
    )

    struttura_responsabile = BlocksField(
        title=_("struttura_responsabile", default="Struttura responsabile"),
        required=False,
        description=_(
            "struttura_responsabile_help",
            default="Nome/link al sito web della struttura che gestisce il"
            " luogo, se questa non è comunale.",
        ),
    )

    orario_pubblico = BlocksField(
        title=_("orario_pubblico", default="Orario per il pubblico"),
        required=False,
        description=_(
            "orario_pubblico_help",
            default="Orario di apertura al pubblico del luogo ed eventuali "
            "regole di accesso (es prenotazione).",
        ),
    )

    # Decisono con Baio di toglierlo: visto il vocabolario, che in realtà sta
    # qui: https://github.com/italia/daf-ontologie-vocabolari-controllati/tree/master/VocabolariControllati/classifications-for-culture/subject-disciplines # noqa
    # riteniamo che possa non fregare nulla a nessuno di questa categorizzazione.
    #  # TODO: aggiungere il vocabolario da https://dataportal.daf.teamdigitale.it/#/vocabularies/subject-disciplines  # noqa
    # # quando ritornano i dati dopo la migrazione, bisognera' vedere dove sono
    # # finiti, link invalido al momento
    # categoria_prevalente = schema.Choice(
    #     title=_("categoria_prevalente", default="Categoria prevalente"),
    #     required=False,
    #     vocabulary="design.plone.contenttypes.Mockup",
    #     missing_value=None,
    #     default=None,
    # )

    # TODO: importare il db del MIBAC, codice DBUnico / ISIL.
    # Non compare nel frontend
    identificativo_mibac = schema.TextLine(
        title=_("identificativo_mibac", default="Identificativo"),
        required=False,
        description=_(
            "identificativo_mibac_help",
            default="Codice identificativo del luogo. Nel MIBAC c'è"
            " il codice del DBUnico per i luoghi della cultura e il codice"
            " ISIL per le biblioteche. Non deve comparire nel"
            " frontend del sito.",
        ),
    )

    # custom fieldsets and order
    form.order_after(nome_alternativo="IBasic.title")

    model.fieldset(
        "descrizione",
        label=_("descrizione_label", default="Descrizione"),
        fields=["descrizione_completa", "elementi_di_interesse"],
    )
    model.fieldset(
        "accesso",
        label=_("accesso_label", default="Modalità di accesso"),
        fields=["modalita_accesso"],
    )

    model.fieldset(
        "categorization",
        fields=["identificativo_mibac"],
    )

    model.fieldset(
        "orari",
        label=_("orari_label", default="Orari di apertura"),
        fields=["orario_pubblico"],
    )
    # TODO: migration script for these commented fields towards PDC
    model.fieldset(
        "contatti",
        label=_("contatti_label", default="Contatti"),
        fields=[
            "struttura_responsabile_correlati",
            "struttura_responsabile",
            # "riferimento_telefonico_struttura",
            # "riferimento_fax_struttura",
            # "riferimento_mail_struttura",
            # "riferimento_pec_struttura",
        ],
    )

    # custom widgets
    form.widget(
        "struttura_responsabile_correlati",
        RelatedItemsFieldWidget,
        vocabulary="plone.app.vocabularies.Catalog",
        pattern_options={
            "selectableTypes": ["UnitaOrganizzativa"],
        },
    )

    # searchabletext indexer
    textindexer.searchable("descrizione_completa")


@implementer(ILuogo)
@adapter(IDexterityContent)
class Luogo(object):
    """ """

    def __init__(self, context):
        self.context = context



# -*- coding: utf-8 -*-
from collective.volto.blocksfield.field import BlocksField
from design.plone.contenttypes import _
from design.plone.contenttypes.interfaces.documento import IDocumento
from plone.app.dexterity import textindexer
from plone.autoform import directives as form
from plone.autoform.interfaces import IFormFieldProvider
from plone.dexterity.interfaces import IDexterityContent
from plone.supermodel import model
from zope.component import adapter
from zope.interface import implementer
from zope.interface import provider


class IDescrizioneEstesaSchema(model.Schema):
    descrizione_estesa = BlocksField(
        title=_("descrizione_estesa", default="Descrizione estesa"),
        required=False,
        description=_(
            "descrizione_estesa_help",
            default="Descrizione dettagliata e completa.",
        ),
    )

    textindexer.searchable("descrizione_estesa")


@provider(IFormFieldProvider)
class IDescrizioneEstesa(IDescrizioneEstesaSchema):
    """ """

    form.order_after(descrizione_estesa="IBasic.description")


@provider(IFormFieldProvider)
class IDescrizioneEstesaServizio(model.Schema):
    """ """

    descrizione_estesa = BlocksField(
        title=_("descrizione_estesa", default="Descrizione estesa"),
        required=False,
        description=_(
            "descrizione_estesa_help",
            default="Descrizione dettagliata e completa.",
        ),
    )

    textindexer.searchable("descrizione_estesa")

    model.fieldset(
        "cose",
        label=_("cose_label", default="Cos'è"),
        fields=["descrizione_estesa"],
    )


@provider(IFormFieldProvider)
class IDescrizioneEstesaDocumento(IDescrizioneEstesaSchema):
    """ """

    descrizione_estesa = BlocksField(
        title=_("descrizione_estesa", default="Descrizione estesa"),
        required=False,
        description=_(
            "descrizione_estesa_help",
            default="Descrizione dettagliata e completa.",
        ),
    )

    model.fieldset(
        "descrizione",
        label=_("descrizione_label", default="Descrizione"),
        fields=["descrizione_estesa"],
    )


@implementer(IDescrizioneEstesa)
@adapter(IDexterityContent)
class DescrizioneEstesa(object):
    """"""

    def __init__(self, context):
        self.context = context


@implementer(IDescrizioneEstesaServizio)
@adapter(IDexterityContent)
class DescrizioneEstesaServizio(object):
    """"""

    def __init__(self, context):
        self.context = context


@implementer(IDescrizioneEstesaDocumento)
@adapter(IDocumento)
class DescrizioneEstesaDocumento(object):
    """"""

    def __init__(self, context):
        self.context = context



# -*- coding: utf-8 -*-
from .upgrades import colors
from .upgrades import logger
from Acquisition import aq_base
from copy import deepcopy
from plone import api


TYPE_TO_TAXONOMIES_MAPPING = {
    "News Item": {
        "tipologia_notizia": {
            "it": {
                "Avviso": "avviso",
                "Comunicato stampa": "comunicato_stampa",
                "Comunicato (stampa)": "comunicato_stampa",
                "Novit\u00e0": "notizia",
                "Notizia": "notizia",
            }
        }
    },
    "Documento": {
        "tipologia_documento": {
            "it": {
                "Accordi tra enti": "accordo_tra_enti",
                "Atti normativi": "atto_normativo",
                "Dataset": "Dataset",
                "Documenti (tecnici) di supporto": "documento_tecnico_di_supporto",
                "Documenti albo pretorio": "documenti_albo_pretorio",
                "Documenti attivit\u00e0 politica": "documento_attivita_politica",
                "Documenti funzionamento interno": "documento_funzionamento_interno",
                "Istanze": "istanza",
                "Modulistica": "modulistica",
            }
        }
    },
    "UnitaOrganizzativa": {
        "tipologia_organizzazione": {
            "it": {
                "Politica": "struttura_politica",
                "Amministrativa": "struttura_amministrativa",
                "Altro": "altra_struttura",
            }
        }
    },
}

TAXONOMIES_MAPPING = {}
for portal_type in TYPE_TO_TAXONOMIES_MAPPING:
    for TAXONOMY in TYPE_TO_TAXONOMIES_MAPPING[portal_type]:
        TAXONOMIES_MAPPING[TAXONOMY] = TYPE_TO_TAXONOMIES_MAPPING[portal_type][TAXONOMY]


def update_taxonomies(context):
    """TODO: documentare"""
    # delete actual index from portal_catalog
    logger.info(
        f"{colors.DARKCYAN} Migrazione delle tassonomie dai vecchi ai nuovi valori {colors.ENDC}"  # noqa
    )
    pc = api.portal.get_tool("portal_catalog")
    for portal_type in TYPE_TO_TAXONOMIES_MAPPING:
        brains = pc(**{"portal_type": portal_type})
        logger.info(
            f"{colors.DARKCYAN} Modifica delle tassonomie per {len(brains)} {portal_type}{colors.ENDC}"  # noqa
        )
        for brain in brains:
            obj = brain.getObject()
            obj_language = getattr(obj, "language", "it") or "it"
            for taxonomy in TYPE_TO_TAXONOMIES_MAPPING[portal_type]:
                mapping = TYPE_TO_TAXONOMIES_MAPPING[portal_type][taxonomy][
                    obj_language
                ]
                old_value = getattr(aq_base(obj), taxonomy, None)
                if type(old_value) == list:  # noqa
                    # this is a sort of race condition.
                    # we already have created ct Documento for attonomina
                    # in case we are using atto di nomina, skip
                    if obj.portal_type != "Documento":
                        raise Exception
                    if obj.id != "atto-di-nomina":
                        raise Exception
                    continue
                else:
                    if old_value and old_value in mapping:
                        new_value = mapping[old_value]
                        if taxonomy == "tipologia_documento":
                            new_value = [new_value]
                        setattr(obj, taxonomy, new_value)
                        logger.info(
                            f"{colors.GREEN} Modifica della tassonomia '{taxonomy}' di {obj.title} da {old_value} a {new_value}{colors.ENDC}"  # noqa
                        )
            obj.reindexObject()


def update_taxonomies_on_blocks(context):
    """
    Code from
    https://github.com/RedTurtle/design.plone.contenttypes/pull/139/files#diff-330d75e9be6e5193ab4622582fe7031d05094784e08aa3ada201a4e3d1642632R33
    """
    # https://www.comune.novellara.re.it/novita/notizie/archivio-notizie

    logger.info(
        f"{colors.DARKCYAN} Update dei blocchi listing basati sulle nuove tassonomie {colors.ENDC}"  # noqa
    )
    brains = api.portal.get_tool("portal_catalog")()
    for index, brain in list(enumerate(brains)):
        item = aq_base(brain.getObject())
        item_language = item.language or "it"
        if not index % 500:
            logger.info(
                f"{colors.DARKCYAN} ({index}/{len(brains)}) Proseguo l'analisi delle pagine {colors.ENDC}"  # noqa
            )
        if getattr(item, "blocks", {}):
            blocks = deepcopy(item.blocks)

            if blocks:
                for block in blocks.values():
                    if block.get("@type", "") == "listing":
                        for query in block.get("querystring", {}).get("query", []):
                            if query["i"] in [
                                "tipologia_notizia",
                                "tipologia_documento",
                                "tipologia_organizzazione",
                            ]:
                                new_values = []
                                for v in query["v"]:
                                    old_value = query["v"]
                                    if (
                                        v
                                        in TAXONOMIES_MAPPING[query["i"]][item_language]
                                    ):
                                        v = TAXONOMIES_MAPPING[query["i"]][
                                            item_language
                                        ][v]
                                    new_values.append(v)
                                query["v"] = new_values
                                logger.info(
                                    f"{colors.GREEN} Modifica della query per '{query['i']}' di {item.title} da {old_value} a {query['v']}{colors.ENDC}"  # noqa
                                )
                item.blocks = blocks
    logger.info(
        f"{colors.DARKCYAN} Terminato l'update dei blocchi {colors.ENDC}"  # noqa
    )



# -*- coding: utf-8 -*-
from collective.taxonomy import PATH_SEPARATOR
from collective.taxonomy.interfaces import ITaxonomy
from design.plone.contenttypes.interfaces import IDesignPloneContenttypesLayer
from plone import api
from plone.dexterity.interfaces import IDexterityContainer
from plone.dexterity.interfaces import IDexterityContent
from plone.restapi.interfaces import ISerializeToJson
from plone.restapi.serializer.dxcontent import (
    SerializeFolderToJson as BaseFolderSerializer,
)
from plone.restapi.serializer.dxcontent import SerializeToJson as BaseSerializer
from zope.component import adapter
from zope.component import queryUtility
from zope.i18n import translate
from zope.interface import implementer


class MetaTypeSerializer(object):
    def get_design_meta_type(self):
        ttool = api.portal.get_tool("portal_types")
        tipologia_notizia = getattr(self.context, "tipologia_notizia", "")
        if self.context.portal_type == "News Item" and tipologia_notizia:
            taxonomy = queryUtility(
                ITaxonomy, name="collective.taxonomy.tipologia_notizia"
            )
            if not taxonomy:
                return tipologia_notizia
            taxonomy_voc = taxonomy.makeVocabulary(self.request.get("LANGUAGE"))
            if isinstance(tipologia_notizia, list):
                token = tipologia_notizia[0]
            else:
                token = tipologia_notizia
            title = taxonomy_voc.inv_data.get(token, None)

            if title and title.startswith(PATH_SEPARATOR):
                return title.replace(PATH_SEPARATOR, "", 1)
        return translate(ttool[self.context.portal_type].Title(), context=self.request)


@implementer(ISerializeToJson)
@adapter(IDexterityContent, IDesignPloneContenttypesLayer)
class SerializeToJson(BaseSerializer, MetaTypeSerializer):
    def __call__(self, version=None, include_items=True):
        result = super(SerializeToJson, self).__call__(
            version=version, include_items=include_items
        )
        result["design_italia_meta_type"] = self.get_design_meta_type()
        return result


@implementer(ISerializeToJson)
@adapter(IDexterityContainer, IDesignPloneContenttypesLayer)
class SerializeFolderToJson(BaseFolderSerializer, MetaTypeSerializer):
    def __call__(self, version=None, include_items=True):
        result = super(SerializeFolderToJson, self).__call__(
            version=version, include_items=include_items
        )
        result["@id"] = self.context.absolute_url()
        result["design_italia_meta_type"] = self.get_design_meta_type()

        if "items_total" not in result:
            # siamo in un sotto-elemento di quello richiesto dalla query.
            #  ritorniamo il numero di elementi totale, senza doverli ritornare
            # effettivamente.
            result["items_total"] = self.context.getFolderContents().actual_result_count
        return result



# -*- coding: utf-8 -*-
from Acquisition import aq_base
from design.plone.contenttypes.interfaces.persona import IPersona
from plone.indexer.decorator import indexer


@indexer(IPersona)
def image_field_indexer(obj):
    """"""
    base_obj = aq_base(obj)

    image_field = ""
    if getattr(base_obj, "foto_persona", False):
        image_field = "foto_persona"
    return image_field


@indexer(IPersona)
def ruolo(obj):
    """
    We read this information from incarico related object
    """
    incarichi = obj.incarichi_persona
    if incarichi:
        # in teoria dovremmo averne uno, ma è consentito averne più di uno.
        # usiamo un keyowrd index per cui in realtà per indicizzare ci interessa
        # poco
        return [x.to_object.title for x in obj.incarichi_persona if not x.isBroken()]
    return []



# -*- coding: utf-8 -*-
from design.plone.contenttypes import _
from plone.app.registry.browser.controlpanel import ControlPanelFormWrapper
from plone.app.registry.browser.controlpanel import RegistryEditForm
from plone.restapi.controlpanels.interfaces import IControlpanel
from zope.interface import Interface
from zope.schema import Bool
from zope.schema import List
from zope.schema import SourceText
from zope.schema import TextLine

import json


class IDesignPloneSettingsControlpanel(IControlpanel):
    """ """


class IDesignPloneSettings(Interface):
    tipologie_notizia = SourceText(
        title=_("tipologie_notizia_label", default="Tipologie Notizia"),
        description=_(
            "tipologie_notizia_help",
            default="Inserisci i valori utilizzabili per le tipologie di una"
            " Notizia. Se il sito è multilingua, puoi inserire valori diversi"
            " a seconda delle lingue del sito.",
        ),
        required=True,
        default=json.dumps({"it": ["Avviso", "Comunicato (stampa)", "Notizia"]}),
    )

    tipologie_unita_organizzativa = SourceText(
        title=_(
            "tipologie_unita_organizzativa_label",
            default="Tipologie Unità Organizzativa",
        ),
        description=_(
            "tipologie_unita_organizzativa_help",
            default="Inserisci i valori utilizzabili per le tipologie di un' "
            "Unità Organizzativa. Se il sito è multilingua, puoi inserire "
            "valori diversi a seconda delle lingue del sito.",
        ),
        required=True,
        default=json.dumps({"it": ["Politica", "Amministrativa", "Altro"]}),
    )

    lead_image_dimension = List(
        title=_(
            "lead_image_dimension_label",
            default="Dimensioni dell'immagine di testata",
        ),
        description=_(
            "lead_image_dimension_help",
            default="Se un tipo di contenuto deve avere una dimensione dell'"
            "immagine di testata particolare, indicarle qui. "
            "Inserire le dimensioni nella forma di esempio "
            "PortalType|900x900",
        ),
        required=True,
        default=[
            "News Item|1920x600",
            "Servizio|1920x600",
            "UnitaOrganizzativa|1920x600",
            "Persona|180x100",
        ],
        value_type=TextLine(),
    )

    search_sections = SourceText(
        title=_("search_sections_label", default="Sezioni ricerca"),
        description=_(
            "search_sections_help",
            default="Inserire una lista di sezioni per la ricerca.",
        ),
        default="",
        required=False,
    )

    show_modified_default = Bool(
        title=_("show_modified_default_label", default="Mostra la data di modifica"),
        description=_(
            "show_modified_default_help",
            default="Questo è il valore di default per decidere se mostrare "
            "o meno la data di modifica nei contenuti che hanno la behavior "
            "abilitata. E' poi possibile sovrascrivere il default nei singoli "
            'contenuti (nel tab "Impostazioni").',
        ),
        default=True,
        required=False,
    )
    show_dynamic_folders_in_footer = Bool(
        title=_("show_dynamic_folders_in_footer_label", default="Footer dinamico"),
        description=_(
            "show_dynamic_folders_in_footer_help",
            default="Se selezionato, il footer verrà popolato automaticamente "
            "con i contenuti di primo livello non esclusi dalla navigazione.",
        ),
        default=True,
        required=False,
    )


class DesignPloneControlPanelForm(RegistryEditForm):
    schema = IDesignPloneSettings
    id = "design-plone-control-panel"
    label = _("Impostazioni Design Plone")


class DesignPloneControlPanelView(ControlPanelFormWrapper):
    """ """

    form = DesignPloneControlPanelForm



# -*- coding: utf-8 -*-
from design.plone.contenttypes import _
from design.plone.contenttypes.controlpanels.geolocation_defaults import (
    IGeolocationDefaults,
)
from plone import api
from plone.restapi.services.types.get import TypesGet as BaseGet
from zope.i18n import translate
from zope.interface import implementer
from zope.publisher.interfaces import IPublishTraverse

import ast


class FieldsetsMismatchError(Exception):
    """Exception thrown when we try to reorder fieldsets, but the order list is
    different from the fieldsets returned from Plone
    """


FIELDSETS_ORDER = {
    "Document": [
        "default",
        "testata",
        "settings",
        "correlati",
        "categorization",
        "dates",
        "ownership",
        "layout",
    ],
    "Documento": [
        "default",
        "descrizione",
        "informazioni",
        "settings",
        "correlati",
        "categorization",
        "dates",
        "ownership",
    ],
    "Event": [
        "default",
        "cose",
        "luogo",
        "date_e_orari",
        "costi",
        "contatti",
        "informazioni",
        "correlati",
        "categorization",
        "dates",
        "settings",
        "ownership",
    ],
    "Incarico": [
        "default",
        "informazioni_compensi",
        "date_e_informazioni",
    ],
    "News Item": [
        "default",
        "dates",
        "correlati",
        "categorization",
        "settings",
        "ownership",
    ],
    "Modulo": [
        "default",
        "settings",
        "correlati",
        "categorization",
        "dates",
        "ownership",
    ],
    "Pagina Argomento": [
        "default",
        "informazioni",
        "correlati",
        "categorization",
        "dates",
        "settings",
        "layout",
        "ownership",
    ],
    "Persona": [
        "default",
        "ruolo",
        "contatti",
        "documenti",
        "informazioni",
        "correlati",
        "categorization",
        "dates",
        "ownership",
        "settings",
    ],
    "PuntoDiContatto": [
        "default",
    ],
    "Servizio": [
        "default",
        "cose",
        "a_chi_si_rivolge",
        "accedi_al_servizio",
        "cosa_serve",
        "costi_e_vincoli",
        "tempi_e_scadenze",
        "casi_particolari",
        "contatti",
        "documenti",
        "link_utili",
        "informazioni",
        "correlati",
        "categorization",
        "settings",
        "ownership",
        "dates",
    ],
    "UnitaOrganizzativa": [
        "default",
        "cosa_fa",
        "struttura",
        "persone",
        "contatti",
        "correlati",
        "categorization",
        "informazioni",
        "settings",
        "ownership",
        "dates",
    ],
    "Venue": [
        "default",
        "descrizione",
        "accesso",
        "dove",
        "orari",
        "contatti",
        "informazioni",
        "settings",
        "correlati",
        "categorization",
    ],
}


@implementer(IPublishTraverse)
class TypesGet(BaseGet):
    def customize_document_schema(self, result):
        moved = ["image", "image_caption", "preview_image"]
        removed = ["preview_caption"]
        for fieldset in result.get("fieldsets", []):
            if fieldset.get("id", "") == "testata":
                fieldset["fields"] = moved + fieldset["fields"]
            if fieldset.get("id", "") == "default":
                fieldset["fields"] = [
                    x for x in fieldset["fields"] if x not in moved and x not in removed
                ]
        return result

    def customize_persona_schema(self, result):
        if "title" in result["properties"]:
            msgid = _("Nome e Cognome", default="Nome e cognome")
            result["properties"]["title"]["title"] = translate(
                msgid, context=self.request
            )
        return result

    def customize_venue_schema(self, result):
        """
        Unico modo per spostare il campo "notes"
        """
        result.get("required").append("description")
        result.get("required").append("street")
        result.get("required").append("city")
        result.get("required").append("zip_code")
        result.get("required").append("geolocation")

        if "properties" in result:
            if "country" in result["properties"]:
                if not result["properties"]["country"].get("default", ""):
                    result["properties"]["country"]["default"] = {
                        "title": "Italia",
                        "token": "380",
                    }
            if "city" in result["properties"]:
                if not result["properties"]["city"].get("default", ""):
                    result["properties"]["city"]["default"] = (
                        api.portal.get_registry_record(
                            "city", interface=IGeolocationDefaults
                        )
                    )
            if "zip_code" in result["properties"]:
                if not result["properties"]["zip_code"].get("default", ""):
                    result["properties"]["zip_code"]["default"] = (
                        api.portal.get_registry_record(
                            "zip_code", interface=IGeolocationDefaults
                        )
                    )

            if "street" in result["properties"]:
                if not result["properties"]["street"].get("default", ""):
                    result["properties"]["street"]["default"] = (
                        api.portal.get_registry_record(
                            "street", interface=IGeolocationDefaults
                        )
                    )

            if "geolocation" in result["properties"]:
                if not result["properties"]["geolocation"].get("default", {}):
                    result["properties"]["geolocation"]["default"] = ast.literal_eval(
                        api.portal.get_registry_record(
                            "geolocation", interface=IGeolocationDefaults
                        )
                    )

        for fieldset in result["fieldsets"]:
            if fieldset.get("id") == "default" and "notes" in fieldset["fields"]:
                fieldset["fields"].remove("notes")
            if fieldset.get("id") == "dove" and "notes" not in fieldset["fields"]:
                fieldset["fields"].append("notes")

        return result

    def customize_versioning_fields_fieldset(self, result):
        """
        Unico modo per spostare i campi del versioning.
        Perché changeNotes ha l'order after="*" che vince su tutto.
        """
        versioning_fields = ["versioning_enabled", "changeNote"]
        for field in versioning_fields:
            found = False
            for fieldset in result["fieldsets"]:
                if fieldset.get("id") == "default" and field in fieldset["fields"]:
                    found = True
                    fieldset["fields"].remove(field)
                if fieldset.get("id") == "settings" and found:
                    fieldset["fields"].append(field)

        return result

    def customize_servizio_schema(self, result):
        result.get("required").append("description")
        return result

    def customize_evento_schema(self, result):
        result.get("required").append("description")
        return result

    def customize_uo_schema(self, result):
        result.get("required").append("description")
        versioning_fields = ["contact_info"]
        for field in versioning_fields:
            for fieldset in result["fieldsets"]:
                if fieldset.get("id") == "contatti" and field in fieldset["fields"]:
                    fieldset["fields"].remove(field)
                    fieldset["fields"].insert(0, field)
        return result

    def customize_news_schema(self, result):
        result.get("required").append("description")
        if self.context.portal_type == "News Item":
            # we are in a news and not in container
            review_state = self.context.portal_workflow.getInfoFor(
                self.context, "review_state"
            )
            if review_state == "published":
                result.get("required").append("effective")

        return result

    def customize_documento_schema(self, result):
        result.get("required").append("description")
        return result

    def reply(self):
        result = super(TypesGet, self).reply()
        if "fieldsets" in result:
            result["fieldsets"] = self.reorder_fieldsets(schema=result)
        pt = self.request.PATH_INFO.split("/")[-1]

        # be careful: result could be dict or list. If list it will not
        # contains title. And this is ok for us.
        pt = self.request.PATH_INFO.split("/")[-1]

        if "title" in result:
            if pt == "Persona":
                result = self.customize_persona_schema(result)
            if pt == "Venue":
                result = self.customize_venue_schema(result)
            if pt == "Document":
                result = self.customize_document_schema(result)
            if pt == "Servizio":
                result = self.customize_servizio_schema(result)
            if pt == "UnitaOrganizzativa":
                result = self.customize_uo_schema(result)
            if pt == "News Item":
                result = self.customize_news_schema(result)
            if pt == "Documento":
                result = self.customize_documento_schema(result)
            if pt == "Event":
                result = self.customize_evento_schema(result)
            result = self.customize_versioning_fields_fieldset(result)
        return result

    def get_order_by_type(self, portal_type):
        return [x for x in FIELDSETS_ORDER.get(portal_type, [])]

    def reorder_fieldsets(self, schema):
        original = schema["fieldsets"]
        pt = self.request.PATH_INFO.split("/")[-1]
        order = self.get_order_by_type(portal_type=pt)
        if not order:
            # no match
            return original
        original_fieldsets = [x["id"] for x in original]

        for fieldset_id in original_fieldsets:
            # if some fieldsets comes from additional addons (not from the
            # base ones), then append them to the order list.
            if fieldset_id not in order:
                order.append(fieldset_id)

        # create a new fieldsets list with the custom order
        new = []
        for id in order:
            for fieldset in original:
                if fieldset["id"] == id and self.fieldset_has_fields(fieldset, schema):
                    new.append(fieldset)
        if not new:
            # no match
            return original
        return new

    def fieldset_has_fields(self, fieldset, schema):
        """
        If a fieldset has all hidden fields (maybe after a schema tweak),
        these are not in the schema data, but are still in fieldset data.
        This happens only in add, because the schema is generate with the parent's context.
        """
        fieldset_fields = fieldset["fields"]

        schema_fields = [x for x in fieldset_fields if x in schema["properties"].keys()]

        return len(schema_fields) > 0
